﻿fn sqr x = x*x

fn checkProductionRenderer = (
	local enabled = false
	if(renderers.production != undefined) then (
		cid = renderers.production.classid
		enabled = ((cid[1] == 1941615238) and (cid[2] == 2012806412) or (cid[1] == 1770671000) and (cid[2] == 1323107829))
	)
	enabled
)

fn getMaxReleaseAsString=(
	local ver=maxversion()
	local result

	if (ver[1]>=10000) then (
		result=(((Integer(ver[1])/Integer(1000))+1998) as String)
	)
	else (
		result="R"+(((ver[1]/1000) as Integer) as String)
	)
	result
)

fn getVRayInstallPath release=(
	local regKeyName="SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall"
	local regKey
	local subKey
	local result
	local replaceIdx=undefined
	try (
		if (registry.openKey HKEY_LOCAL_MACHINE regKeyName accessRights: #readOnly key: &regKey) do (
			local subKeyNames=#()

			registry.getSubKeyNames regKey names: &subKeyNames

			for i = 1 to subKeyNames.count do (
				if (matchPattern subKeyNames[i] pattern:("V-Ray for 3ds*max "+release+" for*")) do (
					registry.getSubKeyName regKey i name: &subKeyName
					if (registry.openKey regKey subKeyName accessRights: #readOnly key: &subKey) do (
						local vrayCmd
						if (registry.queryValue subKey "UninstallString" value: &vrayCmd) do (
							-- Trim install.log's path and doublequotes too
							vrayCmd=substring vrayCmd ((findString vrayCmd "-uninstall=")+12) -1
							vrayCmd=substring vrayCmd 1 ((findString vrayCmd "\"")-1)

							local fInstallLog=openFile (vrayCmd)
							if (fInstallLog!=undefined) do (
								do (
									vrayCmd=readLine fInstallLog
								) while (((findString vrayCmd "\\img2tiledexr.exe")==undefined) OR (eof fInstallLog))
								-- filter vray command
								result=substring vrayCmd ((findString vrayCmd ":\\")-1) -1
								replaceIdx = findString result "img2tiledexr.exe"
								close fInstallLog
							)
						)
						registry.closeKey subKey
					)
				)
			)
			registry.closeKey regKey
		)
	)
	catch (
		if (subKey!=undefined) do (registry.closeKey subKey)
		if (regKey!=undefined) do (registry.closeKey regKey)
	)

	if replaceIdx!=undefined do (
		maketxResult=replace result replaceIdx ("img2tiledexr.exe" as string).count  "maketx.exe"
		if doesfileExist maketxResult then (
			result=maketxResult
			useMakeTx=true
		) else (
			useMakeTx=false
		)
	)
	result
)

-- used to look up materials that have already been converted
orig_mtls=#()
new_mtls=#()

-- used to look up textures that have already been converted
orig_texmaps=#()
new_texmaps=#()

-- used to look up bitmap textures that have already been converted
orig_bitmaptextures=#()
new_hdritexmaps=#()

listOfConvertedNodesHandles=#()

maxver = (maxVersion())[1]

convertToVRay -- Placeholder for the function, otherwise maxscript returns error when executing
					-- the rollout's convert button event if evaluated with the function defined
					-- afterwards.

useMakeTx = false  --Global variable to instruct the converter to tiled EXR to use makeTX or img2TiledExr to perform the conversion

rollout vraySceneConverterRollout "V-Ray scene converter" width: 460 height: 190 (
	label label_message "All scene materials and raytraced shadows will be converted to V-Ray materials and shadows." width:400 height:26 align:#center offset:[0, 10]

	checkbox check_optReflections "Turn on \"trace reflections\" when converting Standard materials" checked:false width:400 align:#center offset:[0, 10]
	checkbox check_optLights "Convert mr Sun/Sky to V-Ray Sun/Sky" checked:false width:400 align:#center offset:[0, 0]
	checkbox check_optStdMtlOpacityToVRayMtlRefract "Convert Standard material opacity to V-Ray material refraction" checked:false width:400 align:#center
	checkbox check_optConvertOnlySelectedNodes "Convert selected nodes only" checked:false width:400 align:#center
	dropDownList ddListCameras selection:1 width:319 align:#left offset:[17, 0]

	button btn_convertStart "Convert" width:60 align:#right across:2 offset:[150, 0]
	button btn_convertCancel "Cancel" width:60 align:#right

	on vraySceneConverterRollout open do (
		if (((maxVersion())[1]) >= 18000) then (
			ddListCameras.items = #("Don't convert standard cameras", "Convert standard cameras to V-Ray Physical Camera", "Convert standard cameras to Max Physical Camera")
		)
		else (
			ddListCameras.items = #("Don't convert standard cameras", "Convert standard cameras to V-Ray Physical Camera")
		)
	)

	on btn_convertStart pressed do (
		convertToVRay  check_optConvertOnlySelectedNodes.checked
		destroyDialog vraySceneConverterRollout
	)

	on btn_convertCancel pressed do (
		destroyDialog vraySceneConverterRollout
	)
)

fn convertFrom_Standard origMtl = (
	r=VRayMtl()
	r.name="vray_"+origMtl.name

	r.texmap_bump=origMtl.bumpMap
	if (r.texmap_bump!=undefined) then (
		r.texmap_bump_on=origMtl.bumpMapEnable
		r.texmap_bump_multiplier=origMtl.bumpMapAmount
	)

	r.texmap_displacement=origMtl.displacementMap
	if (r.texmap_displacement!=undefined) then (
		r.texmap_displacement_on=origMtl.displacementMapEnable
		r.texmap_displacement_multiplier=origMtl.displacementMapAmount
	)

	r.diffuse=origMtl.diffuse
	r.texmap_diffuse=origMtl.diffuseMap
	if (r.texmap_diffuse!=undefined) then (
		r.texmap_diffuse_on=origMtl.diffuseMapEnable
		r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount
	)

	if (origMtl.shaderByName !="Strauss") then (
		r.reflection=origMtl.specular*origMtl.specularLevel*0.01
	)

	if (vraySceneConverterRollout.check_optStdMtlOpacityToVRayMtlRefract.checked) do (
		r.refraction=(Color 255 255 255) * (1.0 - origMtl.opacity*0.01)
	)

	-- unlocked highlights and reflection glossiness no longer supported in V-Ray Next and
	-- that affects reflection and reflection_glossiness initialisation
	local vrMajorVer = 3
	if vrayVersion != undefined do (
		vrMajorVer = (vrayVersion())[1][1] as Integer
	)

	K=2.0^(origMtl.glossiness*0.01*10.0)
	if (origMtl.shaderType==5) then (
		-- phong
		r.brdf_type=0
		if vrMajorVer < 4 do r.reflection=r.reflection*2.0/(K+1.0)
	) else (
		-- blinn
		r.brdf_type=1
		K=K*4
		if vrMajorVer < 4 do r.reflection=r.reflection*8.0/(K+1.0)
	)

	if vrMajorVer > 3 then (
		r.reflection_glossiness=1.0-(1.0/(K+1.0))^(1.0/3.5)
	)
	else (
		r.reflection_lockGlossiness=false
		r.hilight_glossiness=1.0-(1.0/(K+1.0))^(1.0/3.5)
		r.reflection_glossiness=1.0
	)
	r.option_traceReflection=vraySceneConverterRollout.check_optReflections.checked

	r.texmap_opacity=origMtl.opacityMap
	if (r.texmap_opacity!=undefined) then (
		r.texmap_opacity_on=origMtl.opacityMapEnable
		r.texmap_opacity_multiplier=origMtl.opacityMapAmount
	)

	r
)

fn convertFrom_Raytrace origMtl = (
	r=VRayMtl()
	r.name="vray_"+origMtl.name

	r.texmap_bump=origMtl.bumpMap
	if (r.texmap_bump!=undefined) then (
		r.texmap_bump_on=origMtl.bumpMapEnable
		r.texmap_bump_multiplier=origMtl.bumpMapAmount
	)

	r.texmap_displacement=origMtl.displacementMap
	if (r.texmap_displacement!=undefined) then (
		r.texmap_displacement_on=origMtl.displacementMapEnable
		r.texmap_displacement_multiplier=origMtl.displacementMapAmount
	)

	r.diffuse=origMtl.diffuse
	r.texmap_diffuse=origMtl.diffuseMap
	if (r.texmap_diffuse!=undefined) then (
		r.texmap_diffuse_on=origMtl.diffuseMapEnable
		r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount
	)

	r.reflection=origMtl.spec__color*origMtl.specular_level*0.01*0.5

	K=2.0^(origMtl.glossiness*0.01*10.0)
	if (origMtl.shaderType==1) then (
		-- phong
		r.brdf_type=0
		r.reflection=r.reflection*2.0/(K+1.0)
	) else (
		-- blinn
		r.brdf_type=1
		K=K*4
		r.reflection=r.reflection*8.0/(K+1.0)
	)

	if (origMtl.reflect.r>r.reflection.r) then r.reflection.r=origMtl.reflect.r
	if (origMtl.reflect.g>r.reflection.g) then r.reflection.g=origMtl.reflect.g
	if (origMtl.reflect.b>r.reflection.b) then r.reflection.b=origMtl.reflect.b

	r.reflection_glossiness=1.0-(1.0/(K+1.0))^(1.0/3.5)

	if ((origMtl.reflect.r+origMtl.reflect.g+origMtl.reflect.b)/3.0<1e-6) then r.option_traceReflection=false

	r.texmap_reflection=origMtl.reflectionMap
	if (r.texmap_reflection!=undefined) then (
		r.texmap_reflection_on=origMtl.reflectionMapEnable
		r.texmap_reflection_multiplier=origMtl.reflectionMapAmount
	)

	r.refraction=origMtl.transparecy
	r.refraction_ior=origMtl.Index_of_Refraction

	r.texmap_refraction=origMtl.transparencyMap
	if (r.texmap_refraction!=undefined) then (
		r.texmap_refraction_on=origMtl.transparencyMapEnable
		r.texmap_refraction_multiplier=origMtl.transparencyMapAmount
	)

	r
)

fn convertFrom_fR_Advanced origMtl = (
	r=VRayMtl()
	r.name="vray_"+origMtl.name

	-- diffuse
	r.diffuse=origMtl.diffuseColor
	r.texmap_diffuse=origMtl.diffuseMap
	r.texmap_diffuse_on=origMtl.useDiffuseMap
	r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount

	-- bump map
	r.texmap_bump=origMtl.bumpMap
	r.texmap_bump_on=origMtl.useBumpMap
	r.texmap_bump_multiplier=origMtl.bumpMapAmount

	-- displacement
	r.texmap_displacement=origMtl.MTDMap
	r.texmap_displacement_on=origMtl.useMTDMap
	r.texmap_displacement_multiplier=origMtl.MTDMapAmount

	-- reflection
	r.reflection=origMtl.reflectFilter
	r.texmap_reflection=origMtl.reflectMap
	r.texmap_reflection_on=origMtl.useReflectMap
	r.texmap_reflection_multiplier=origMtl.reflectMapAmount

	r.reflection_glossiness=1.0-sqr (origMtl.reflectGlossy*0.01)
	r.texmap_reflectionGlossiness=origMtl.reflectBlurryMap
	r.texmap_reflectionGlossiness_on=origMtl.useReflectBlurryMap
	r.texmap_reflectionGlossiness_multiplier=origMtl.reflectBlurryMapAmount

	r.reflection_subdivs=sqrt origMtl.reflectSamples
	r.reflection_fresnel=origMtl.useFresnel
	r.reflection_useInterpolation=origMtl.useReflBlur

	-- refraction
	r.refraction=origMtl.refractFilter
	r.texmap_refraction=origMtl.refractMap
	r.texmap_refraction_on=origMtl.useRefractMap
	r.texmap_refraction_multiplier=origMtl.refractMapAmount

	r.refraction_ior=origMtl.refractIOR
	r.texmap_refractionIOR=origMtl.refractIORMap
	r.texmap_refractionIOR_on=origMtl.useRefractIORMap
	r.texmap_refractionIOR_multiplier=origMtl.refractIORMapAmount

	r.refraction_glossiness=1.0-sqr (origMtl.refractGlossy*0.01)
	r.texmap_refractionGlossiness=origMtl.refractBlurryMap
	r.texmap_refractionGlossiness_on=origMtl.useRefractBlurryMap
	r.texmap_refractionGlossiness_multiplier=origMtl.refractBlurryMapAmount

	r.refraction_subdivs=sqrt origMtl.refractSamples
	r.refraction_useInterpolation=origMtl.useRefrBlur

	r
)

fn convertFrom_fR_Metal origMtl = (
	r=VRayMtl()
	r.name="vray_"+origMtl.name

	r.diffuse=origMtl.diffuseColor
	r.texmap_diffuse=origMtl.diffuseMap
	r.texmap_diffuse_on=origMtl.diffuseMapEnable
	r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount

	r.reflection=origMtl.reflectColor
	r.texmap_reflection=origMtl.reflectLevelMap
	r.texmap_reflection_on=origMtl.reflectLevelMapEnable
	r.texmap_reflection_multiplier=origMtl.reflectLevelMapAmount

	r
)

fn convertFrom_fR_Glass origMtl = (
	r=VRayMtl()
	r.name="vray_"+origMtl.name

	r.diffuse=origMtl.diffuseColor
	r.texmap_diffuse=origMtl.diffuseMap
	r.texmap_diffuse_on=origMtl.diffuseMapEnable
	r.texmap_diffuse_multiplier=origMtl.diffuseMapAmount

	r.reflection=origMtl.reflectColor
	r.reflection_fresnel=origMtl.useFresnel
	r.texmap_reflection=origMtl.reflectMap
	r.texmap_reflection_on=origMtl.reflectMapEnable
	r.texmap_reflection_multiplier=origMtl.reflectMapAmount

	r.refraction=origMtl.refractColor
	r.refraction_ior=origMtl.refractIOR
	r.texmap_refraction=origMtl.refractMap
	r.texmap_refraction_on=origMtl.refractMapEnable
	r.texmap_refraction_multiplier=origMtl.refractMapAmount

	r.refraction_affectShadows=true
	r.refraction_affectAlpha=1

	r
)

fn createVRayMtl=undefined
fn createVRayTexture=undefined

fn convertFromBlend orig_mtl = (
	r=VRayBlendMtl()
	r.name="vray_"+orig_mtl.name

	r.baseMtl = createVRayMtl orig_mtl.map1
	r.coatMtl[1] = createVRayMtl orig_mtl.map2
	r.blend[1]=(orig_mtl.mixAmount*0.01)*(color 255 255 255)
	if orig_mtl.mask != undefined then (
		r.texmap_blend[1] = createVRayTexture orig_mtl.mask
		if orig_mtl.maskEnabled == off then r.texmap_blend_multiplier[1]=0
	)

	r
)

fn convertFromShellac orig_mtl = (
	r=VRayBlendMtl()
	r.name="vray_"+orig_mtl.name
	r.additiveMode=true

	r.baseMtl = createVRayMtl orig_mtl.shellacMtl1
	r.coatMtl[1] = createVRayMtl orig_mtl.shellacMtl2
	r.blend[1]=(orig_mtl.shellacColorBlend*0.01)*(color 255 255 255)

	r
)

fn convertFromDoubleSided orig_mtl = (
	r=VRay2SidedMtl()
	r.name="vray_"+orig_mtl.name

	r.frontMtl = createVRayMtl orig_mtl.material1
	r.backMtl = createVRayMtl orig_mtl.material2
	r.translucency=orig_mtl.translucency*0.01*(color 255 255 255)
	r.backMtlOn=orig_mtl.map2Enabled

	r
)

fn convertFrom_Architectural orig_mtl = (
	r=VRayMtl()
	r.name="vray_"+orig_mtl.name

	-- diffuse component
	r.diffuse=orig_mtl.diffuse
	r.texmap_diffuse=orig_mtl.diffuseMap
	if (r.texmap_diffuse!=undefined) then (
		r.texmap_diffuse_on=orig_mtl.diffuseMapEnable
		r.texmap_diffuse_multiplier=orig_mtl.diffuseMapAmount
	)

	-- specular component
	r.reflection=(color 255 255 255)
	r.texmap_reflection=orig_mtl.shininessMap
	r.reflection_fresnel=true
	r.reflection_glossiness=(orig_mtl.shininess*0.01)*0.25+0.75

	-- refraction
	r.refraction=orig_mtl.transparency*0.01*(color 255 255 255)
	r.refraction_ior=orig_mtl.ior

	-- if there is luminance, wrap the material into a VRayBlendMtl and add a VRayLightMtl on top of it
	if orig_mtl.luminance>0 then (
		local l=VRayLightMtl()
		l.color=orig_mtl.diffuse
		l.multiplier=orig_mtl.luminance
		l.opacity_texmap=VRayColor()
		l.opacity_texmap.color=(color 0 0 0)

		local m=VRayBlendMtl()
		m.baseMtl=r
		m.coatMtl[1]=l
		m.blend[1]=(color 255 255 255)

		r=m
	)

	-- if there is translucency, wrap the material into a VRay2SidedMtl
	if orig_mtl.translucency>0 then (
		local m=VRay2SidedMtl()
		m.frontMtl=r
		m.translucency=orig_mtl.translucency*0.01*(color 255 255 255)
		r=m
	)

	r
)

fn convertFrom_ArchAndDesign orig_mtl = (
	r=VRayMtl()
	r.name="vray_"+orig_mtl.name

	-- diffuse component
	r.diffuse=orig_mtl.diff_color*orig_mtl.diff_weight
	r.texmap_diffuse=orig_mtl.diff_color_map
	if (r.texmap_diffuse!=undefined) then (
		r.diffuse=(color 0 0 0)
		r.texmap_diffuse_on=orig_mtl.diff_color_map_on
		r.texmap_diffuse_multiplier=orig_mtl.diff_weight*100
	)

	-- reflection
	r.reflection=orig_mtl.refl_color*orig_mtl.refl_weight
	r.texmap_reflection=orig_mtl.refl_color_map
	if (r.texmap_reflection!=undefined) then (
		r.reflection=(color 0 0 0)
		r.texmap_reflection_on=orig_mtl.refl_color_map_on
		r.texmap_reflection_multiplier=orig_mtl.refl_weight*100
	)

	r.reflection_glossiness=orig_mtl.refl_gloss
	r.texmap_reflectionGlossiness=orig_mtl.refl_gloss_map
	if (r.texmap_reflectionGlossiness!=undefined) then (
		r.reflection_glossiness=0.0
		r.texmap_reflectionGlossiness_on=orig_mtl.refl_gloss_map_on
	)

	-- Fresnel options
	if (orig_mtl.refl_func_fresnel==true) then ( -- use Fresnel
		r.reflection_fresnel=true
	) else ( -- use falloff curve
		local tcolor=Falloff()
		if (r.texmap_reflection==undefined) then (
			tcolor.color1=orig_mtl.refl_color*orig_mtl.refl_func_low
			tcolor.color2=orig_mtl.refl_color*orig_mtl.refl_func_high
		) else (
			tcolor.color1=(color 1.0 1.0 1.0)*orig_mtl.refl_func_low
			tcolor.color2=(color 1.0 1.0 1.0)*orig_mtl.refl_func_high

			local m=VRayCompTex()
			m.sourceA=r.texmap_reflection
			m.sourceB=tcolor
			m.operator=3
			tcolor=m
		)

		r.texmap_reflection=tcolor
		r.texmap_reflection_on=true
		r.texmap_reflection_multiplier=orig_mtl.refl_weight*100
		r.reflection=(color 0 0 0)
	)

	-- refraction/opacity
	r.refraction_ior=orig_mtl.refr_ior
	r.refraction_affectShadows=not orig_mtl.opts_do_refractive_caustics
	r.refraction_affectAlpha=(if orig_mtl.opts_propagate_alpha then 1 else 0)
	if (orig_mtl.opts_1sided==on) then ( -- opacity
		if (orig_mtl.refr_color_map!=undefined) then ( -- there is a transparency texture
			local tcolor=Output()
			tcolor.map1=orig_mtl.refr_color_map
			tcolor.output.invert=true
			tcolor.output_amount=orig_mtl.refr_weight

			r.texmap_opacity=tcolor
		) else ( -- no transparency texture, just color
			local c=orig_mtl.refr_color
			if (orig_mtl.refr_weight>0 and (c.r>0 or c.g>0 or c.b>0)) then ( -- there is some transparency
				local tcolor=VRayColor()
				tcolor.color=(color (1.0-orig_mtl.refr_color.r) (1.0-orig_mtl.refr_color.g) (1.0-orig_mtl.refr_color.b))
				r.texmap_opacity=tcolor
			)
		)
	) else ( -- refraction
		r.refraction=orig_mtl.refr_color*orig_mtl.refr_weight
		r.texmap_refraction=orig_mtl.refr_color_map
		if (r.texmap_refraction!=undefined) then (
			r.refraction=(color 0 0 0)
			r.texmap_refraction_on=orig_mtl.refr_color_map_on
			r.texmap_refraction_multiplier=orig_mtl.refr_weight*100
		)
	)

	r
)

fn convertFrom_CoronaMtl orig_mtl = (
	r=VRayMtl()
	r.name="vray_"+orig_mtl.name

	--Diffuse
	r.diffuse = orig_mtl.levelDiffuse * orig_mtl.colorDiffuse
	r.texmap_diffuse=orig_mtl.texmapDiffuse
	if r.texmap_diffuse != undefined then(
		r.texmap_diffuse_on = orig_mtl.texmapOnDiffuse
		r.texmap_diffuse_multiplier = orig_mtl.mapAmountDiffuse
	)

	--Reflection
	r.reflection = orig_mtl.levelReflect  * orig_mtl.colorReflect
	r.texmap_reflection = orig_mtl.texmapReflect
	if r.texmap_reflection != undefined then(
		r.texmap_reflection_on = orig_mtl.texmapOnReflect
		r.texmap_reflection_multiplier = orig_mtl.mapamountReflect
	)

	r.reflection_fresnel = true
	if orig_mtl.fresnelIor == orig_mtl.Ior then
	(
		r.reflection_lockIOR = true
		r.reflection_ior = orig_mtl.ior
		r.refraction_ior = orig_mtl.ior
	)else(
		r.reflection_lockIOR = false
		r.reflection_ior = orig_mtl.fresnelIor
	)

	r.texmap_reflectionIOR = orig_mtl.texmapFresnelIor
	if r.texmap_reflectionIOR != undefined then(
		r.texmap_reflectionIOR_on = orig_mtl.texmapOnFresnelIor
		r.texmap_reflectionIOR_multiplier = orig_mtl.mapamountFresnelIor
	)

	r.reflection_glossiness = orig_mtl.reflectGlossiness
	r.texmap_reflectionGlossiness = orig_mtl.texmapReflectGlossiness
	if r.texmap_reflectionGlossiness != undefined then(
		r.texmap_reflectionGlossiness_on = orig_mtl.texmapOnReflectGlossiness
		r.texmap_reflectionGlossiness_multiplier = orig_mtl.mapamountReflectGlossiness
	)

	--Anisotropy
	r.anisotropy = orig_mtl.anisotropy
	r.texmap_anisotropy = orig_mtl.texmapReflectAnisotropy
	if r.texmap_anisotropy != undefined then(
		r.texmap_anisotropy_on = orig_mtl.texmapOnAnisotropy
		r.texmap_anisotropy_multiplier = orig_mtl.mapamountAnisotropy
	)
	r.anisotropy_rotation = orig_mtl.anisotropyRotation
	r.texmap_anisotropy_rotation = orig_mtl.texmapReflectAnisotropyRotation
	if r.texmap_anisotropy_rotation != undefined then(
		r.texmap_anisotropy_rotation_on = orig_mtl.texmapOnAnisotropyRotation
		r.texmap_anisotropy_rotation_multiplier = orig_mtl.mapamountAnisotropyRotation
	)

	--Refraction
	r.refraction = orig_mtl.levelRefract*orig_mtl.colorRefract
	r.texmap_refraction = orig_mtl.texmapRefract
	if r.texmap_refraction != undefined then(
		r.texmap_refraction_on = orig_mtl.texmapOnRefract
		r.texmap_refraction_multiplier = orig_mtl.mapamountRefract
	)

	r.refraction_ior = orig_mtl.ior
	r.texmap_refractionIOR = orig_mtl.texmapIor
	if r.texmap_refractionIOR != undefined then(
		r.texmap_refractionIOR_on = orig_mtl.texmapOnIor
		r.texmap_refractionIOR_multiplier = orig_mtl.mapamountIor
	)

	r.refraction_glossiness = orig_mtl.refractGlossiness
	r.texmap_refractionGlossiness = orig_mtl.texmapRefractGlossiness
	if r.texmap_refractionGlossiness != undefined then(
		r.texmap_refractionGlossiness_on = orig_mtl.texmapOnRefractGlossiness
		r.texmap_refractionGlossiness_multiplier = orig_mtl.mapamountRefractGlossiness
	)

	r.refraction_affectShadows = not orig_mtl.caustics
	if  orig_mtl.thin do r.refraction_ior = 1.0

	--Opacity
	r.texmap_opacity = orig_mtl.texmapOpacity
	if r.texmap_opacity != undefined then(
		r.texmap_opacity_on = orig_mtl.texmapOnOpacity
		r.texmap_opacity_multiplier = orig_mtl.mapamountOpacity
	)

	--Bumpmap
	r.texmap_bump = orig_mtl.texmapBump
	if r.texmap_bump != undefined then(
		r.texmap_bump_on = orig_mtl.texmapOnBump
		r.texmap_bump_multiplier = 100.0*orig_mtl.mapamountBump
	)

	--Absorption
	r.refraction_fogMult  = if orig_mtl.absorptionDistance == 0.0 then 0.0 else 1.0/orig_mtl.absorptionDistance
	r.refraction_fogColor  = orig_mtl.absorptionColor

	--Displacement
	r.texmap_displacement = orig_mtl.texmapDisplace
	if r.texmap_displacement != undefined then(
		r.texmap_displacement_on = orig_mtl.texmapOnDisplacement
		r.texmap_displacement_multiplier = (orig_mtl.displacementMaximum - orig_mtl.displacementMinimum) --(Displace_amount) : worldUnits
	)

	--Self illumination
	r.selfIllumination_multiplier = orig_mtl.levelSelfIllum
	r.selfIllumination = if orig_mtl.levelSelfIllum == 0.0 then black else orig_mtl.colorSelfIllum
	r.texmap_self_illumination = orig_mtl.texmapSelfIllum
	if r.texmap_self_illumination != undefined then(
		r.texmap_self_illumination_on = orig_mtl.texmapOnSelfIllum
		r.texmap_self_illumination_multiplier = orig_mtl.mapamountSelfIllum
	)

	-- Translucency
	if orig_mtl.levelTranslucency > 0.0 then(
		newVRay2SidedMtl = VRay2SidedMtl name:("vray "+orig_mtl.name)
		newVRay2SidedMtl.frontMtl = r

			if orig_mtl.texmapTranslucency == undefined then(
				newVRay2SidedMtl.texmap_translucency_multiplier = 100.0*orig_mtl.levelTranslucency
				newVRay2SidedMtl.Translucency = orig_mtl.colorTranslucency
			)
			if orig_mtl.texmapTranslucency != undefined and orig_mtl.texmapOnTranslucency == true then(
				theMix = Mix map1: orig_mtl.texmapTranslucency color2: orig_mtl.colorTranslucency mixAmount: orig_mtl.mapamountTranslucency
				newVRay2SidedMtl.texmap_translucency_multiplier = 100.0*orig_mtl.levelTranslucency
				newVRay2SidedMtl.Translucency = white
				newVRay2SidedMtl.texmap_translucency = theMix
			)
		return newVRay2SidedMtl
	) else return r
)

fn convertFrom_CoronaLayeredMtl orig_mtl = (
	r=VRayBlendMtl()
	r.name="vray_"+orig_mtl.name

	r.baseMtl = orig_mtl.baseMtl
	for iLayer = 1 to orig_mtl.layers.count - 1 do (
		r.coatMtl[iLayer] = orig_mtl.layers[iLayer]
		r.texmap_blend[iLayer] = orig_mtl.mixmaps[iLayer]
		r.blend[iLayer].value = orig_mtl.amounts[iLayer] * 255.0 /100.0
		if orig_mtl.amounts[iLayer] == 0.0 and hasProperty r #coatMtl_enable then (
			r.coatMtl_enable[iLayer] = false
		)
	)

	return r
)

fn convertFrom_CoronaLightMtl orig_mtl = (
	r=VRayLightMtl()
	r.name="vray_"+orig_mtl.name

	r.color = orig_mtl.color
	r.multiplier = orig_mtl.intensity
	r.directLight_on = orig_mtl.emitLight

	if orig_mtl.texmap != undefined then(
		r.texmap_on = orig_mtl.texmapOn
		r.texmap = orig_mtl.texmap
	)
	if orig_mtl.opacityTexmap != undefined then(
		r.opacity_texmap_on = orig_mtl.opacityTexmapOn
		r.opacity_texmap = orig_mtl.opacityTexmap
	)
	return r
)

fn convertFrom_CoronaRaySwitchMtl orig_mtl = (
	r = VRayOverrideMtl()
	r.name="vray_"+orig_mtl.name

	r.baseMtl = orig_mtl.directMtl
	r.giMtl = orig_mtl.giMtl
	r.reflectMtl = orig_mtl.reflectMtl
	r.refractMtl = orig_mtl.refractMtl

	return r
)

fn convertFrom_CoronaVolumeMtl orig_mtl = (
	vr=VRayScatterVolume()
	vr.name="vray_"+orig_mtl.name

	vr.overall_color = orig_mtl.absorptionColor
	vr.sss_color = orig_mtl.scatteringColor
	vr.scatter_color = orig_mtl.absorptionColor
	vr.phase_function = orig_mtl.scatteringDirectionality
	vr.scatter_radius = orig_mtl.absorptionDist
	if orig_mtl.singleBounce then vr.bounces = 1

	if orig_mtl.absorptionColorTexmap != undefined and orig_mtl.absorptionColorTexmapOn == true then (
		vr.texmap_overall_color = orig_mtl.absorptionColorTexmap
		vr.texmap_scatter_color = orig_mtl.absorptionColorTexmap
	)

	if orig_mtl.scatteringColorTexmap != undefined and orig_mtl.scatteringColorTexmapOn == true then (
		vr.texmap_sss_color = orig_mtl.scatteringColorTexmap
	)

	if orig_mtl.emissionDist > 0 then (
		vrLightMtl=VRayLightMtl()
		vrLightMtl.name = "vraylight "+orig_mtl.name
		vrLightMtl.color = orig_mtl.emissionColor
		vrLightMtl.multiplier = (1.0f / orig_mtl.emissionDist)

		vrColorMap = VRayColor()
		vrColorMap.color = (Color 0 0 0)
		vrLightMtl.opacity_texmap = vrColorMap

		vrBlendMtl = VRayBlendMtl()
		vrBlendMtl.name = "vrayblend "+orig_mtl.name
		vrBlendMtl.baseMtl = vr
		vrBlendMtl.coatMtl[1] = vrLightMtl
		return vrBlendMtl
	)

	return vr
)

fn convertFrom_CoronaShadowCatcherMtl orig_mtl = (
	vrwrap=VRayMtlWrapper()
	vrwrap.name="vray_"+orig_mtl.name

	vrwrap.matteSurface = true
	vrwrap.alphaContribution = case orig_mtl.alphaMode of (
		1: 1.0f
		2: -1.0f
		3: 0.0f
		default: -1.0f
	)
	vrwrap.matte_shadows = true
	vrwrap.matte_shadowsAffectAlpha = true
	vrwrap.matte_shadowBrightness = orig_mtl.shadowAmount

	vr=VRayMtl()
	vr.diffuse = orig_mtl.backplateColor
	if orig_mtl.backplateTexmap != undefined then (
		vr.texmap_diffuse = orig_mtl.backplateTexmap
		vr.texmap_diffuse_on = true
	)

	--Reflection
	vr.reflection = orig_mtl.reflectLevel  * orig_mtl.colorReflect
	vr.texmap_reflection = orig_mtl.reflectColorTexmap

	vr.reflection_fresnel = true
	vr.reflection_lockIOR = false
	vr.reflection_ior = orig_mtl.reflectIor
	vr.reflection_glossiness = orig_mtl.reflectGlossiness

	vr.texmap_bump = orig_mtl.bump
	if vr.texmap_bump != undefined then(
		vr.texmap_bump_on = true
		vr.texmap_bump_multiplier = 100.0*orig_mtl.bumpAmount
	)

	vrwrap.baseMtl = vr
	return vrwrap
)

fn convertFrom_CoronaHairMtl orig_mtl = (
	vr=VRayHairNextMtl()
	vr.name="vray_"+orig_mtl.name

	vr.melanin = orig_mtl.melanin
	vr.random_melanin = orig_mtl.randomizeMelanin
	vr.texmap_melanin_on = orig_mtl.texmapOnMelanin
	vr.texmap_melanin_multiplier = orig_mtl.mapamountMelanin
	vr.texmap_melanin = orig_mtl.texmapMelanin
	vr.pheomelanin = orig_mtl.pheomelanin

	vr.dye_color = orig_mtl.levelTint * orig_mtl.colorTint
	vr.texmap_dye_color_on = orig_mtl.texmapOnTint
	vr.texmap_dye_color_multiplier = orig_mtl.mapamountTint
	if (orig_mtl.levelTint == 1) then (
		vr.texmap_dye_color = orig_mtl.texmapTint
	) else if (orig_mtl.levelTint == 0) then (
		vr.texmap_dye_color_on = off
		vr.texmap_dye_color = orig_mtl.texmapTint
	) else (
		outTexDye = output()
		outTexDye.map1 = orig_mtl.texmapTint
		outTexDye.output.output_amount = orig_mtl.levelTint
		vr.texmap_dye_color = outTexDye
	)

	vr.opacity = orig_mtl.levelOpacity * orig_mtl.colorOpacity
	vr.texmap_opacity_on = orig_mtl.texmapOnOpacity
	vr.texmap_opacity_multiplier = orig_mtl.mapamountOpacity
	if (orig_mtl.levelOpacity == 1) then (
		vr.texmap_opacity = orig_mtl.texmapOpacity
	) else if (orig_mtl.levelOpacity == 0) then (
		vr.texmap_opacity_on = off
		vr.texmap_opacity = orig_mtl.texmapOpacity
	) else (
		outTexOpacity = output()
		outTexOpacity.map1 = orig_mtl.texmapOpacity
		outTexOpacity.output.output_amount = orig_mtl.levelOpacity
		vr.texmap_opacity = outTexOpacity
	)

	vr.glossiness = orig_mtl.glossiness
	vr.texmap_glossiness_on = orig_mtl.texmapOnGlossiness
	vr.texmap_glossiness_multiplier = orig_mtl.mapamountGlossiness
	vr.texmap_glossiness = orig_mtl.texmapGlossiness
	vr.primary_glossiness_boost = orig_mtl.colorlessGlossiness
	vr.texmap_primary_glossiness_boost_on = orig_mtl.texmapOnColorlessGlossiness
	vr.texmap_primary_glossiness_boost_multiplier = orig_mtl.mapamountColorlessGlossiness
	vr.texmap_primary_glossiness_boost = orig_mtl.texmapColorlessGlossiness
	vr.highlight_shift = orig_mtl.highlightShift

	vr.secondary_tint = orig_mtl.coloredSpecular
	vr.texmap_secondary_tint = orig_mtl.texmapColoredSpecular
	vr.texmap_secondary_tint_on = orig_mtl.texmapOnColoredSpecular
	vr.texmap_secondary_tint_multiplier = orig_mtl.mapamountColoredSpecular

	vr.primary_tint = orig_mtl.colorlessSpecular
	vr.texmap_primary_tint = orig_mtl.texmapColorlessSpecular
	vr.texmap_primary_tint_on = orig_mtl.texmapOnColorlessSpecular
	vr.texmap_primary_tint_multiplier = orig_mtl.mapamountColorlessSpecular

	vr.transmission_tint = orig_mtl.transmission
	vr.texmap_transmission_tint = orig_mtl.texmapTransmission
	vr.texmap_transmission_tint_on = orig_mtl.texmapOnTransmission
	vr.texmap_transmission_tint_multiplier = orig_mtl.mapamountTransmission

	vr.diffuse_amount = orig_mtl.levelDiffuse
	vr.diffuse_color = orig_mtl.colorDiffuse
	vr.texmap_diffuse_color = orig_mtl.texmapDiffuse
	vr.texmap_diffuse_color_on = orig_mtl.texmapOnDiffuse
	vr.texmap_diffuse_color_multiplier = orig_mtl.mapAmountDiffuse

	vr.softness = orig_mtl.softness
	vr.texmap_softness = orig_mtl.texmapSoftness
	vr.texmap_softness_on = orig_mtl.texmapOnSoftness
	vr.texmap_softness_multiplier = orig_mtl.mapamountSoftness

	vr.ior = orig_mtl.ior
	vr.texmap_ior = orig_mtl.texmapIor
	vr.texmap_ior_on = orig_mtl.texmapOnIor
	vr.texmap_ior_multiplier = orig_mtl.mapamountIor

	vr
)

fn convertFrom_CoronaSkinMtl orig_mtl = (
	vr=VRayALSurfaceMtl()
	vr.name="vray_"+orig_mtl.name

	vr.sss_density_scale = orig_mtl.sssRadiusScale

	vr.opacity = orig_mtl.opacityLevel*orig_mtl.opacityColor
	if (orig_mtl.texmapOpacityColor != undefined) do (
		vr.opacity_texture_on = orig_mtl.texmapOnOpacityColor
		vr.opacity_texture_multiplier = orig_mtl.mapamountOpacityColor
		if (orig_mtl.opacityLevel == 1) then (
			vr.opacity_texture = orig_mtl.texmapOpacityColor
		) else if (orig_mtl.opacityLevel == 0) then (
			vr.opacity_texture_on = off
			vr.opacity_texture = orig_mtl.texmapOpacityColor
		) else (
			outTexOpacity = output()
			outTexOpacity.map1 = orig_mtl.texmapOpacityColor
			outTexOpacity.output.output_amount = orig_mtl.opacityLevel
			vr.opacity_texture = outTexOpacity
		)
	)

	vr.displacement_texture = orig_mtl.texmapDisplace
	vr.displacement_texture_on = orig_mtl.texmapOnDisplace
	--vr.displacement_texture_multiplier = displaceMinimum?, displaceWaterLevel?, displaceMaximum?

	vr.bump_texture = orig_mtl.texmapBump
	vr.bump_texture_on = orig_mtl.texmapOnBump
	vr.bump_texture_multiplier = orig_mtl.mapamountBump

	vr.diffuse_color = orig_mtl.skinColor
	vr.diffuse_strength = orig_mtl.skinLevel
	vr.diffuse_color_texture = orig_mtl.texmapSkinColor
	vr.diffuse_color_texture_on = orig_mtl.texmapOnSkinColor
	vr.diffuse_color_texture_multiplier = orig_mtl.mapamountSkinColor
	vr.sss_mix = orig_mtl.sssAmount
	vr.sss_mix_texture = orig_mtl.texmapSssAmount
	vr.sss_mix_texture_on = orig_mtl.texmapOnSssAmount
	vr.sss_mix_texture_multiplier = orig_mtl.mapamountSssAmount
	vr.sss_mode = orig_mtl.sssMode
	if orig_mtl.sssMode == 0 then vr.sss_mode = 1
	else vr.sss_mode = 0

	--SSS1
	vr.sss1_color = orig_mtl.sss1Color
	vr.sss1_color_texture = orig_mtl.texmapSss1Color
	vr.sss1_color_texture_on = orig_mtl.texmapOnSss1Color
	vr.sss1_color_texture_multiplier = orig_mtl.mapamountSss1Color
	vr.sss1_weight = orig_mtl.sss1Weight
	vr.sss1_weight_texture = orig_mtl.texmapSss1Weight
	vr.sss1_weight_texture_on = orig_mtl.texmapOnSss1Weight
	vr.sss1_weight_texture_multiplier = orig_mtl.mapamountSss1Weight
	vr.sss1_radius = orig_mtl.sss1Radius
	vr.sss1_radius_texture = orig_mtl.texmapSss1Radius
	vr.sss1_radius_texture_on = orig_mtl.texmapOnSss1Radius
	vr.sss1_radius_texture_multiplier = orig_mtl.mapamountSss1Radius

	--SSS2
	vr.sss2_color = orig_mtl.sss2Color
	vr.sss2_color_texture = orig_mtl.texmapSss2Color
	vr.sss2_color_texture_on = orig_mtl.texmapOnSss2Color
	vr.sss2_color_texture_multiplier = orig_mtl.mapamountSss2Color
	vr.sss2_weight = orig_mtl.sss2Weight
	vr.sss2_weight_texture = orig_mtl.texmapSss2Weight
	vr.sss2_weight_texture_on = orig_mtl.texmapOnSss2Weight
	vr.sss2_weight_texture_multiplier = orig_mtl.mapamountSss2Weight
	vr.sss2_radius = orig_mtl.sss2Radius
	vr.sss2_radius_texture = orig_mtl.texmapSss2Radius
	vr.sss2_radius_texture_on = orig_mtl.texmapOnSss2Radius
	vr.sss2_radius_texture_multiplier = orig_mtl.mapamountSss2Radius

	--SSS3
	vr.sss3_color = orig_mtl.sss3Color
	vr.sss3_color_texture = orig_mtl.texmapSss3Color
	vr.sss3_color_texture_on = orig_mtl.texmapOnSss3Color
	vr.sss3_color_texture_multiplier = orig_mtl.mapamountSss3Color
	vr.sss3_weight = orig_mtl.sss3Weight
	vr.sss3_weight_texture = orig_mtl.texmapSss3Weight
	vr.sss3_weight_texture_on = orig_mtl.texmapOnSss3Weight
	vr.sss3_weight_texture_multiplier = orig_mtl.mapamountSss3Weight
	vr.sss3_radius = orig_mtl.sss3Radius
	vr.sss3_radius_texture = orig_mtl.texmapSss3Radius
	vr.sss3_radius_texture_on = orig_mtl.texmapOnSss3Radius
	vr.sss3_radius_texture_multiplier = orig_mtl.mapamountSss3Radius

	--Reflection1
	vr.reflect1_color = orig_mtl.reflection1Color
	vr.reflect1_color_texture = orig_mtl.texmapReflection1Color
	vr.reflect1_color_texture_on = orig_mtl.texmapOnReflection1Color
	vr.reflect1_color_texture_multiplier = orig_mtl.mapamountReflection1Color
	vr.reflect1_strength = orig_mtl.reflection1Level
	vr.reflect1_roughness = 1 - orig_mtl.reflection1Glossiness
	if (orig_mtl.texmapReflection1Glossiness != undefined) do (
		outRefl1RoughTex = output()
		outRefl1RoughTex.map1 = orig_mtl.texmapReflection1Glossiness
		outRefl1RoughTex.output.invert = on
		vr.reflect1_roughness_texture = outRefl1RoughTex
		vr.reflect1_roughness_texture_on = orig_mtl.texmapOnReflection1Glossiness
		vr.reflect1_roughness_texture_multiplier = orig_mtl.mapamountReflection1Glossiness
	)
	vr.reflect1_ior = orig_mtl.reflection1Ior
	vr.reflect1_ior_texture = orig_mtl.texmapReflection1Ior
	vr.reflect1_ior_texture_on = orig_mtl.texmapOnReflection1Ior
	vr.reflect1_ior_texture_multiplier = orig_mtl.mapamountReflection1Ior

	--Reflection2
	vr.reflect2_color = orig_mtl.reflection2Color
	vr.reflect2_color_texture = orig_mtl.texmapReflection2Color
	vr.reflect2_color_texture_on = orig_mtl.texmapOnReflection2Color
	vr.reflect2_color_texture_multiplier = orig_mtl.mapamountReflection2Color
	vr.reflect2_strength = orig_mtl.reflection2Level
	vr.reflect2_roughness = 1 - orig_mtl.reflection2Glossiness
	if (orig_mtl.texmapReflection2Glossiness != undefined) do (
		outRefl2RoughTex = output()
		outRefl2RoughTex.map1 = orig_mtl.texmapReflection2Glossiness
		outRefl2RoughTex.output.invert = on
		vr.reflect2_roughness_texture = outRefl2RoughTex
		vr.reflect2_roughness_texture_on = orig_mtl.texmapOnReflection2Glossiness
		vr.reflect2_roughness_texture_multiplier = orig_mtl.mapamountReflection2Glossiness
	)
	vr.reflect2_ior = orig_mtl.reflection2Ior
	vr.reflect2_ior_texture = orig_mtl.texmapReflection2Ior
	vr.reflect2_ior_texture_on = orig_mtl.texmapOnReflection2Ior
	vr.reflect2_ior_texture_multiplier = orig_mtl.mapamountReflection2Ior

	vr
)

fn convertFrom_CoronaColorTex inTexture = (
	vrayC = VRayColor name:("vray_"+inTexture.name) alpha:inTexture.alpha rgb_multiplier: inTexture.multiplier
	if inTexture.method == 0 do (
		vrayC.color_mode = 0
		vrayC.color = inTexture.color
	)
	if inTexture.method == 1 do (
		vrayC.color_mode = 0
		vrayC.red = inTexture.colorHdr[1]
		vrayC.green = inTexture.colorHdr[2]
		vrayC.blue = inTexture.colorHdr[3]
	)
	if inTexture.method == 2 do (
		vrayC.color_mode = 1
		vrayC.temperature = inTexture.temperature
	)
	if inTexture.method == 3 do (
		vrayC.color_mode = 0
		local strHexColor = inTexture.hexColor
		if (strHexColor.count == 6 or strHexColor.count == 7) then (
			local startVal = 1
			if ((findString strHexColor "#") == 1) do startVal = 2

			local subStrHexColor = "0x" + (substring strHexColor startVal 2)
			local rColor = (subStrHexColor as float)
			startVal = startVal +2
			subStrHexColor = "0x" + (substring strHexColor startVal 2)
			local gColor = (subStrHexColor as float)
			startVal = startVal +2
			subStrHexColor = "0x" + (substring strHexColor startVal 2)
			local bColor = (subStrHexColor as float)

			if (rColor!=undefined and gColor!=undefined and bColor!=undefined) then (
				vrayC.color = (color rColor gColor bColor)
			) else (
				vrayC.color = (color 0.0f 0.0f 0.0f)
			)
		) else (
			vrayC.color = (color 255.0f 0.0f 0.0f)
		)
	)
	return  vrayC
)

fn convertFrom_CoronaMultiMapTex inTexture = (
	vrayMST = VRayMultiSubTex name:("vray_"+inTexture.name)
	if vrayMST.texmap_num > inTexture.items do (
		local removeItems = vrayMST.texmap_num - inTexture.items
		vrayMST.deleteSubtex (inTexture.items + 1) removeItems
	)

	if vrayMST.texmap_num < inTexture.items do (
		local addItems = inTexture.items - vrayMST.texmap_num
		vrayMST.addSubtex addItems
	)

	if inTexture.mode == 5 do vrayMST.from_id = 1
	if inTexture.mode == 2 do vrayMST.from_id = 6

	for i = 1 to inTexture.items do (
		vrayMST.texmap_color[i] = inTexture.colors[i]
		vrayMST.texmap[i] = inTexture.texmaps[i]
	)

	return vrayMST
)

fn convertFrom_CoronaMixTex inTexture = (
	vrayCT = VRayCompTex sourceA: inTexture.texmapBottom sourceB: inTexture.texmapTop name:("vray_"+inTexture.name)
	vrayCT.operator = case inTexture.mixOperation of (
		0: 0 -- Add
		1: 1 -- subtract
		2: 3 -- multiply
		3: 4 -- divide
		4: 5 -- minimum
		5: 6 -- maximum
		8: 2 -- difference
		default:0
		)
	return vrayCT
)

fn convertFrom_CoronaAoTex inTexture = (
	vrayDt = VRayDirt occluded_color:inTexture.colorOccluded unoccluded_color:inTexture.colorUnoccluded \
	radius:inTexture.maxDistance texmap_radius:inTexture.texmapDistance texmap_occluded_color:inTexture.texmapOccluded \
	texmap_unoccluded_color:inTexture.texmapUnoccluded name:("vray_"+inTexture.name)

	if (hasProperty inTexture "invert") then (
		vrayDt.invert_normal = inTexture.invert
	)
	else if (hasProperty inTexture "normalMode") then (
		vrayDt.invert_normal = case of (
			(inTexture.normalMode == 0):false
			(inTexture.normalMode == 1):true
			(inTexture.normalMode == 2):false
			default:false)
	)

	if inTexture.excludeMode == 1 and inTexture.excludeList.count != 0 do vrayDt.excludeList = inTexture.excludeList
	if inTexture.excludeMode == 3 do vrayDt.consider_same_object_only = true

	vrayDt.texmap_radius = if inTexture.texmapDistance != undefined do inTexture.texmapDistance
	vrayDt.texmap_occluded_color = if inTexture.texmapOccluded != undefined do inTexture.texmapOccluded
	vrayDt.texmap_unoccluded_color = if inTexture.texmapUnoccluded != undefined do inTexture.texmapUnoccluded

	return vrayDt
)

fn convertFrom_CoronaNormalTex inTexture = (
	vrayNM = VRayNormalMap normal_map:inTexture.normalMap name:("vray_"+inTexture.name)
	try( vrayNM.map_channel =  inTexture.normalMap.coords.mapChannel
		)catch()

	return vrayNM
)

fn convertFrom_CoronaRoundEdges inTexture = (
	vrRoundEdges = VRayEdgesTex name:("vray_"+inTexture.name) roundedCorners_radius:inTexture.radius \
	roundedCorners_sameObjectOnly:(inTexture.excludeMode == 1)
)

fn convertFrom_CoronaDistance inTexture = (
	vrDistTex = VRayDistanceTex  name:("vray_"+inTexture.name)

	vrDistTex.objects = inTexture.nodes
	vrDistTex.texmap_far_on = inTexture.texmapFarOn
	vrDistTex.far_color = inTexture.colorFar
	vrDistTex.texmap_far = inTexture.texmapFar
	vrDistTex.texmap_near_on = inTexture.texmapNearOn
	vrDistTex.near_color = inTexture.colorNear
	vrDistTex.texmap_near = inTexture.texmapNear
	vrDistTex.distance = inTexture.distanceFar
	vrDistTex.tex_inside_separate_on = inTexture.useInside
	vrDistTex.texmap_distance_on = inTexture.distanceScaleTexmapOn
	vrDistTex.texmap_distance = inTexture.distanceScaleTexmap

	if (inTexture.useInside == on) then (
		vrDistTex.inside_color = inTexture.colorInside
		vrDistTex.texmap_inside = inTexture.texmapInside
		vrDistTex.texmap_inside_on = inTexture.texmapInsideOn
	)

	vrDistTex
)

fn convertFrom_CoronaTriplanar inTexture = (
	vrTriplanarTex = VRayTriplanarTex name:("vray_"+inTexture.name)

	vrTriplanarTex.texture_mode = if inTexture.onlyTexmapX == on then 0 else 1

	if inTexture.texmapX !=undefined and inTexture.texmapXOn == on then
		vrTriplanarTex.texture = inTexture.texmapX
	if inTexture.texmapY !=undefined and inTexture.texmapYOn == on then
		vrTriplanarTex.texture_y = inTexture.texmapY
	if inTexture.texmapZ !=undefined and inTexture.texmapZOn == on then
		vrTriplanarTex.texture_z = inTexture.texmapZ

	vrTriplanarTex.size = inTexture.scale
	vrTriplanarTex.blend = inTexture.blend

	case inTexture.space of (
		0: (
			vrTriplanarTex.space = 0
		)
		1: (
			vrTriplanarTex.space = 1
			vrTriplanarTex.reference_node = undefined
		)
		2: (
			vrTriplanarTex.space = 1
			vrTriplanarTex.reference_node = inTexture.referenceNode
		)
	)
	vrTriplanarTex.frame_offset = inTexture.offset
	vrTriplanarTex.texture_rotation = inTexture.rotation

	vrTriplanarTex
)

fn convertFrom_CoronaBitmap inTexture = (
	local texHDRI=VRayHDRI()
	texHDRI.name="vray_"+inTexture.name

	if inTexture.filename != "" and inTexture.filename != undefined then (
		texHDRI.HDRIMapName = inTexture.filename
	)

	if inTexture.enviroMapping == 4 then texHDRI.mapType = 3

	case inTexture.interpolation of (
		0: (texHDRI.interpType = 0; texHDRI.filterMode = 0)
		1: (texHDRI.interpType = 1; texHDRI.filterMode = 0)
		2: texHDRI.filterMode = 1
	)

	-- Gamma handling
	if (maxversion())[1] >= 16000 and inTexture.gamma != -1 then(
		texHDRI.gamma = 1.0/inTexture.gamma
	)

	-- uv channel, environment mode
	if inTexture.uvwChannel == 0 then (
		texHDRI.coords.mapping = 1
	)else(
		texHDRI.coords.mapChannel = inTexture.uvwChannel
	)

	-- offset, tiling/scale, rotation, rw mapscale
	texHDRI.coords.realWorldScale = inTexture.realWorldScale

	texHDRI.coords.U_Offset = inTexture.uvwOffset[1]
	texHDRI.coords.V_Offset = inTexture.uvwOffset[2]

	if inTexture.realWorldScale then (
		texHDRI.coords.realWorldWidth = inTexture.uvwScale[1]
		texHDRI.coords.realWorldHeight = inTexture.uvwScale[2]
	)else(
		texHDRI.coords.U_Tiling = inTexture.uvwScale[1]
		texHDRI.coords.V_Tiling = inTexture.uvwScale[2]
	)

	case inTexture.tilingU of (
		0: (texHDRI.coords.U_Mirror = false; texHDRI.coords.U_Tile = false)
		2: (texHDRI.coords.U_Mirror = true; texHDRI.coords.U_Tile = false)
		1: (texHDRI.coords.U_Mirror = false; texHDRI.coords.U_Tile = true)
	)
	case inTexture.tilingV of (
		0: (texHDRI.coords.V_Mirror = false; texHDRI.coords.V_Tile = false)
		2: (texHDRI.coords.V_Mirror = true; texHDRI.coords.V_Tile = false)
		1: (texHDRI.coords.V_Mirror = false; texHDRI.coords.V_Tile = true)
	)
	texHDRI.coords.W_angle = inTexture.wAngle

	-- crop/place
	texHDRI.cropplace_on = inTexture.clippingOn
	texHDRI.cropplace_mode = inTexture.clippingMode
	texHDRI.cropplace_u = inTexture.clippingU
	texHDRI.cropplace_v = inTexture.clippingV
	texHDRI.cropplace_width = inTexture.clippingWidth
	texHDRI.cropplace_height = inTexture.clippingHeight

	-- alpha source, mono channel out, rgb channel out
	texHDRI.alphaSource = inTexture.alphaSource
	texHDRI.monoOutput = inTexture.monoChannelOutput
	texHDRI.rgbOutput = inTexture.rgbChannelOutput

	texHDRI.coords.blur = inTexture.filteringBlur

	--Convert output parameters including curve control
	replaceInstances texHDRI.output (copy inTexture.output)

	texHDRI
)

fn convertFrom_AutodeskBitmap texmap=(
	if (texmap!=undefined and (classof texmap)==Autodesk_Map) then (
		local texHDRI=VRayHDRI()
		texHDRI.name="vray_"+texmap.name

		if (IsProperty texmap "Parameters_Source") and (undefined != texmap.Parameters_Source) do
		(
			texHDRI.HDRIMapName = texmap.Parameters_Source.filename
		)
		if (IsProperty texmap "Parameters_Brightness") and (undefined != texmap.Parameters_Brightness) do
		(
			texHDRI.output.RGB_Level = texmap.Parameters_Brightness / 100
		)

		if (IsProperty texmap "Parameters_Invert_Image") and (undefined != texmap.Parameters_Invert_Image) do
		(
			texHDRI.output.invert = texmap.Parameters_Invert_Image
		)

		texHDRI.coords.mappingType = 0 --Texture
		texHDRI.coords.mapping = 0 --Explicit Map Channel
		texHDRI.coords.realWorldScale = true

		if (IsProperty texmap "Position_X") and (undefined != texmap.Position_X) do
		(
			texHDRI.UVGen.u_offset = texmap.Position_X
			if(texmap.Position_X.controller != undefined) do texHDRI.UVGen.u_offset = texmap.Position_X.controller
		)

		if (IsProperty texmap "Position_Y") and (undefined != texmap.Position_Y) do
		(
			texHDRI.UVGen.v_offset = texmap.Position_Y
			if(texmap.Position_Y.controller != undefined) do texHDRI.UVGen.v_offset.controller = texmap.Position_Y.controller
		)

		if (IsProperty texmap "Scale_Width") and (undefined != texmap.Scale_Width) do
		(
			texHDRI.UVGen.realWorldWidth = texmap.Scale_Width
			if(texmap.Scale_Width.controller != undefined) do texHDRI.UVGen.realWorldWidth.controller = texmap.Scale_Width.controller
		)

		if (IsProperty texmap "Scale_Height") and (undefined != texmap.Scale_Height) do
		(
			texHDRI.UVGen.realWorldHeight = texmap.Scale_Height
			if(texmap.Scale_Height.controller != undefined) do texHDRI.UVGen.realWorldHeight.controller = texmap.Scale_Height.controller
		)

		--According to the documentation the rotation is done on the Z axis, so rotation is on the W axis
		if (IsProperty texmap "Position_Rotation") and (undefined != texmap.Position_Rotation) do
		(
			texHDRI.UVGen.W_Angle = texmap.Position_Rotation
			if(texmap.Position_Rotation.controller != undefined) do texHDRI.UVGen.W_Angle.controller = texmap.Position_Rotation.controller
		)
		texHDRI.UVGen.V_Angle = 0.0
		texHDRI.UVGen.U_Angle = 0.0

		texHDRI.UVGen.U_Mirror = false
		texHDRI.UVGen.V_Mirror = false
		if (IsProperty texmap "Repeat_Horizontal") and (undefined != texmap.Repeat_Horizontal) do
		(
			texHDRI.UVGen.U_Tile = texmap.Repeat_Horizontal
		)
		if (IsProperty texmap "Repeat_Vertical") and (undefined != texmap.Repeat_Vertical) do
		(
			texHDRI.UVGen.V_Tile = texmap.Repeat_Vertical
		)

		if (IsProperty texmap "Advanced_Parameters_Blur") and (undefined != texmap.Advanced_Parameters_Blur) do
		(
			texHDRI.UVGen.blur = texmap.Advanced_Parameters_Blur
			if(texmap.Advanced_Parameters_Blur.controller != undefined) do texHDRI.UVGen.blur.controller = texmap.Advanced_Parameters_Blur.controller
		)

		if (IsProperty texmap "Advanced_Parameters_Blur_Offset") and (undefined != texmap.Advanced_Parameters_Blur_Offset) do
		(
			texHDRI.UVGen.Blur_Offset = texmap.Advanced_Parameters_Blur_Offset
			if(texmap.Advanced_Parameters_Blur_Offset.controller != undefined) do texHDRI.UVGen.Blur_Offset.controller = texmap.Advanced_Parameters_Blur_Offset.controller
		)

		if (IsProperty texmap "Advanced_Parameters_Map_Channel") and (undefined != texmap.Advanced_Parameters_Map_Channel) do
		(
			texHDRI.UVGen.mapChannel = texmap.Advanced_Parameters_Map_Channel
		)

		texHDRI.mapType=4 /*The mapping type is determined by the Coordinates section.*/
		texHDRI.color_space=3 /*The gamma is controlled from 3ds Max (customize -> preferences -> Gamma and LUT -> Input Gamma)*/

		texHDRI
	)
	else (
		undefined
	)
)

fn isVRayCompatible mtl_classid = (
	-- native V-Ray materials
	if mtl_classid==VRayMtl then return true
	if mtl_classid==VRay2SidedMtl then return true
	if mtl_classid==VRayBlendMtl then return true
	if mtl_classid==VRayFastSSS then return true
	if mtl_classid==VRayLightMtl then return true
	if mtl_classid==VRayMtlWrapper then return true

	-- supported 3dsmax materials
	if mtl_classid==Multimaterial then return true
	if mtl_classid==Arch___Design__mi then return true

	false
)

fn isVRayCompatibleTex tex_classid = (
	-- native V-Ray textures
	compatibleTexClassIDs = #(VRayBmpFilter, VRayBump2Normal, VRayColor, VRayColor2Bump, VRayCompTex, VRayCurvature, VRayDirt, VRayDistanceTex, VRayEdgesTex, \
										  VRayFakeFresnelTex, VRayGLSLTex, VRayHairInfoTex, VRayHDRI, VRayICC, VRayLut, VRayMap, VRayMultiSubTex, VRayOCIO, VRayOSLTex, \
										  VRayParticleTex, VRayPtex, VRaySamplerInfoTex, VRaySky, VRaySoftbox, VRayTriplanarTex, VRayUserColor, VRayUserScalar)
    result = findItem compatibleTexClassIDs tex_classid
	if result > 0 then return true
	return false
)

fn createVRayTexture inTexture= (
	-- check if this is a texture; if not, just return the value
	if ((superclassof inTexture)!=textureMap) then return inTexture

	-- check if we have already converted this texture; if yes, just return
	-- the already converted one
	local i=findItem orig_texmaps inTexture
	if i>0 then return new_texmaps[i]

	-- recurse into sub-textures, if any
	local sub_count=getNumSubTexmaps inTexture
	if sub_count>0 then (
		local sub_tex = undefined
		for i=1 to sub_count do (
			sub_tex = getSubTexmap inTexture i
			if(sub_tex != undefined) then
				setSubTexmap inTexture i (createVRayTexture sub_tex)
		)
	)

	-- get the texture class id
	cls=classof inTexture

	-- if this is already a V-Ray compatible material, return the same one
	if (isVRayCompatibleTex cls) then return inTexture

	-- try to convert
	format "  %\n" inTexture
	local new_tex=case cls of (
		Normal_Bump: VRayNormalMap name:inTexture.name normal_map:inTexture.normal_map normal_map_on:inTexture.map1on normal_map_multiplier:inTexture.mult_spin \
											bump_map:inTexture.bump_map bump_map_on:inTexture.map2on bump_map_multiplier:inTexture.bump_spin \
											flip_red:inTexture.flipred flip_green:inTexture.flipgreen swap_red_and_green:inTexture.swap_rg
		Autodesk_Map: convertFrom_AutodeskBitmap inTexture
		CoronaOutput: if inTexture.inputTexmap != undefined do inTexture.inputTexmap
		CoronaRaySwitch: VRayColor color:red name:("vray_"+inTexture.name)
		CoronaFrontBack: VRayColor color:inTexture.frontColor name:("vray_"+inTexture.name)
		CoronaColor: convertFrom_CoronaColorTex inTexture
		CoronaMultiMap: convertFrom_CoronaMultiMapTex inTexture
		CoronaMix: convertFrom_CoronaMixTex inTexture
		CoronaAo: convertFrom_CoronaAoTex inTexture
		CoronaNormal: convertFrom_CoronaNormalTex inTexture
		coronasky: VRaySky name:("vray_"+inTexture.name)
		CoronaWire: VRayEdgesTex edgesColor:inTexture.edgeColor widthType:(if inTexture.widthMode == 1 then 0 else 1) thickness:inTexture.edgeWidthWorld \
											pixelWidth:inTexture.edgeWidthPixels name:("vray_"+inTexture.name)

		CoronaDistance: convertFrom_CoronaDistance inTexture
		CoronaRoundEdges: convertFrom_CoronaRoundEdges inTexture
		CoronaTriplanar: convertFrom_CoronaTriplanar inTexture
		CoronaBitmap: convertFrom_CoronaBitmap inTexture
		default: inTexture
	)

	-- if converted, add to the list of converted textures
	if new_tex!=inTexture then (
		append orig_texmaps inTexture
		append new_texmaps new_tex
	)

	-- return the new texture
	new_tex
)

fn createVRayMtl orig_mtl = (
	-- check if this is a material; if not, just return the value
	if ((superclassof orig_mtl)!=material) then return orig_mtl

	-- convert sub-textures of this material
	local sub_texs_count=getNumSubTexmaps orig_mtl
	if sub_texs_count>0 then (
		local sub_tex = undefined
		for i=1 to sub_texs_count do (
			sub_tex = getSubTexmap orig_mtl i
			if(sub_tex != undefined) then
				setSubTexmap orig_mtl i (createVRayTexture sub_tex)
		)
	)

	-- check if we have already converted this material; if yes, just return
	-- the already converted one
	local i=findItem orig_mtls orig_mtl
	if i>0 then return new_mtls[i]

	-- recurse into sub-materials, if any
	local sub_count=getNumSubMtls orig_mtl
	if sub_count>0 then (
		local sub_mtl = undefined
		for i=1 to sub_count do (
			sub_mtl=getSubMtl orig_mtl i
			if(sub_mtl != undefined) then
				setSubMtl orig_mtl i (createVRayMtl sub_mtl)
		)
	)

	-- get the material class id
	cls=classof orig_mtl

	-- if this is already a V-Ray compatible material, return the same one
	if (isVRayCompatible cls) then (
		--If the VRayMtl comes from old scenes with unlocked hilight glossiness
		if cls==VRayMtl and orig_mtl.reflection_lockGlossiness==off then (
			orig_mtl.hilight_glossiness = orig_mtl.reflection_glossiness
			orig_mtl.texmap_hilightGlossiness = undefined
			orig_mtl.reflection_lockGlossiness = on
		)
		return orig_mtl
	)

	-- otherwise, try to convert
	format "  %\n" orig_mtl
	local new_mtl=case cls of (
		StandardMaterial: convertFrom_Standard orig_mtl
		RaytraceMaterial: convertFrom_Raytrace orig_mtl
		Architectural: convertFrom_Architectural orig_mtl

		fR_Advanced: convertFrom_fR_Advanced orig_mtl
		fR_Metal: convertFrom_fR_Metal orig_mtl
		fR_Glass: convertFrom_fR_Glass orig_mtl

		Blend: convertFromBlend orig_mtl
		Shellac: convertFromShellac orig_mtl
		DoubleSided: convertFromDoubleSided orig_mtl

		CoronaMtl: convertFrom_CoronaMtl orig_mtl
		CoronaLayeredMtl: convertFrom_CoronaLayeredMtl orig_mtl
		CoronaLightMtl: convertFrom_CoronaLightMtl orig_mtl
		CoronaRaySwitchMtl: convertFrom_CoronaRaySwitchMtl orig_mtl
		CoronaPortalMtl: VRayMtl name:("vray_" + orig_mtl.name)
		CoronaVolumeMtl: convertFrom_CoronaVolumeMtl orig_mtl
		CoronaHairMtl: convertFrom_CoronaHairMtl orig_mtl
		CoronaSkinMtl: convertFrom_CoronaSkinMtl orig_mtl


		-- Do not convert Arch&Design materials - V-Ray supports them directly
		-- Arch___Design__mi: convertFrom_ArchAndDesign orig_mtl

		default: orig_mtl
	)

	-- if converted, add to the list of converted materials
	if new_mtl!=orig_mtl then (
		append orig_mtls orig_mtl
		append new_mtls new_mtl
	)

	-- return the new material
	new_mtl
)

fn replaceMtlNode node = (
	if isValidNode node then (
		local m=createVRayMtl node.material
		node.material=m
		format "    %\n" node.name
	)
	for i=1 to node.children.count do (
		replaceMtlNode node.children[i]
	)
)

-- convert all scene materials to VRay materials
fn mtlsToVRay selectedOnly= (
	format "Converting scene materials...\n"
	orig_mtls=#()
	new_mtls=#()
	orig_texmaps=#()
	new_texmaps=#()
	if selectedOnly then (
		format "Converting scene materials for selected nodes...\n"
		local numNodes = selection.count
		for iNode=1 to numNodes do (
			print selection[iNode]
			replaceMtlNode selection[iNode]
		)
	) else (
		replaceMtlNode rootNode

		format "Converting Material Editor materials...\n"
		for i=1 to 24 do (
			local m=createVRayMtl meditmaterials[i]
			meditmaterials[i]=m
		)
	)
	orig_mtls=#()
	new_mtls=#()
	orig_texmaps=#()
	new_texmaps=#()
	format "Done\n"
)

fn deselectSMENodes = (
	if (maxver < 13000) do return undefined

	local numViews = sme.GetNumViews()
	for iView=1 to numViews do (
		local currentView = sme.GetView iView
		local numNodes = currentView.GetNumNodes()
		for iNode=1 to numNodes do (
			local currentNode = currentView.GetNode iNode
			currentNode.selected = false
		)
	)
)

fn selectSMENodeByRef objRef = (
	if (maxver < 13000) do return undefined

	local numViews = sme.GetNumViews()
	for iView=1 to numViews do (
		local currentView = sme.GetView iView
		local nodeByRef = currentView.GetNodeByRef objRef
		if (nodeByRef != undefined) then (
			nodeByRef.selected = true
			print nodeByRef.name
		)
	)
)

fn deleteSMESelectedNodesInEachView = (
	if (maxver < 13000) do return undefined

	local numViews = sme.GetNumViews()
	for iView=1 to numViews do (
		local currentView = sme.GetView iView
		currentView.DeleteSelection()
	)
)

fn showMtlSlotInSME activeSlot= (

	local m = getMeditMaterial activeSlot
	local maxver = (maxVersion())[1]
	if (maxver >= 13000 and MatEditor.mode == #advanced) then (
		local vrmatViewIdx = sme.GetViewByName "VRayMtlView"
		if (vrmatViewIdx == 0) then (
			vrmatViewIdx = sme.CreateView "VRayMtlView"
		)
		local vrmatView = sme.GetView vrmatViewIdx

		if (vrmatView != undefined) then (
			vrmatView.CreateNode m [0,0]
		)
	)
)

fn warningToSelectGeometry theOperationName= (
	format "V-Ray Toolbar WARNING!\n"
	format "In order to create % you have to select geometry first!\n" theOperationName
)

fn shadowsNode node = (
	if (isValidNode node) then (
		scls=superclassof node
		if (scls==light) then (
			try (
				olds=node.shadowGenerator
				cls=classof olds
				if (cls!=shadowMap and cls!=VRayShadow) then (
					format "  %\n" node
					news=VRayShadow()
					node.shadowGenerator=news
				)
			)
			catch ()
		)
	)

	for i=1 to node.children.count do (
		shadowsNode node.children[i]
	)
)

-- convert all shadow types except shadow maps to VRay shadows
fn shadowsToVRay selectedOnly = (
	format "Converting shadows...\n"
	if selectedOnly then (
		format "Converting shadows for selected nodes...\n"
		local numNodes = selection.count
		for iNode=1 to numNodes do (
			print selection[iNode]
			shadowsNode selection[iNode]
		)
	) else (
		shadowsNode rootNode
	)
	format "Done\n"
)

fn replaceCoronaLightNodeVRayLight cLt = (
	print cLt.name
	local theVRL=CreateInstance VRayLight name:(cLt.name+"_VRayLight") on:cLt.on castShadows:cLt.occludeOtherLights invisible:(not cLt.visibleDirectly) \
	affect_reflections:cLt.visibleRefl color_mode:cLt.colorMode  color:cLt.color targeted:cLt.targeted

	if cLt.colorMode == 1 do theVRL.color_temperature = cLt.blackbodyTemp
	if cLt.colorMode == 2 do theVRL.texmap_on = true
	if cLt.texmap != undefined do theVRL.texmap = cLt.texmap

	if cLt.targeted == true do theVRL.target_distance = cLt.targetDistance

	theVRL.lightDistribution = cLt.directionality

	--Include/Exclude
	if (cLt.excludeList != undefined and cLt.excludeList.count !=0) do (
		theVRL.inclExclType = 1
		if cLt.excludeIncludeMod then theVRL.includeList = cLt.excludeList else theVRL.excludeList = cLt.excludeList
	)

	--Color/ Intensity
	theVRL.normalizeColor = case of
	(
		(cLt.intensityUnits == 0):0
		(cLt.intensityUnits == 1):1
		default:0
	)

	theVRL.multiplier = cLt.intensity

	--shape
	case cLt.shape  of
	(
		1: ( --Plane
			theVRL.type = 0
			theVRL.size0 = cLt.width/2.0
			theVRL.size1 = cLt.height/2.0
		)
		0: ( --Sphere
			theVRL.type = 2
			theVRL.size0 = cLt.width
		)
		2: ( --Disc
			theVRL.type = 4
			theVRL.size0 = cLt.width/2.0
		)
		3: ( --Cylinder
			theVRL.type = 2
			theVRL.size0 = cLt.width
		)
	)

	instCount = InstanceMgr.GetInstances cLt &arrInst
	for i=1 to instCount do
		arrInst[i].name = (arrInst[i].name+"_VRayLight")
	replaceInstances cLt theVRL
	theVRL
)

--here input param : 'ver' have to be presented as <number>.<number>.{<number>}
fn checkForPreviousCoronaVersion ver = (
	res = false
	if CoronaRenderer != undefined do (
		local currVer = CoronaRenderer.getCoronaVersion()
		local parsedCurrVer = filterstring currVer " ,."
		local intCurrVer = ((parsedCurrVer[1]+parsedCurrVer[2]) as integer)
		local parsedVer = filterstring ver " ,."
		local intVer = ((parsedVer[1]+parsedVer[2]) as integer)
		if intCurrVer != undefined and intVer != undefined then
			res = (intCurrVer < intVer)
	)
	return res
)

fn replaceCoronaSunLightNodeVRayLight coronaSLNode = (
	local sunObj = undefined
	local targetObj = undefined
	local previousVersion = checkForPreviousCoronaVersion("1.2.1")

	if previousVersion then (
		sunObj = VRaySun name:(coronaSLNode.name+"_VRaySun") enabled:coronaSLNode.on intensity_multiplier:coronaSLNode.intensity size_multiplier:coronaSLNode.sizeMultiplier invisible:(not coronaSLNode.visibleDirectly)
		targetObj = dummy name:(coronaSLNode.name+"_VRaySun.Target") pos:coronaSLNode.pos  --then target
		targetObj.boxsize = [2,2,2]
		targetObj.lookat = sunObj --this makes the target for the sunn out of dummy object
		sunObj.pos = coronaSLNode.pos + coronaSLNode.displaySize*[sin coronaSLNode.height, 0.0, cos coronaSLNode.height] -- xy
		sunObj.pos = [coronaSLNode.pos.x , coronaSLNode.pos.y, sunObj.pos.z]  + [cos coronaSLNode.sunRotation , sin coronaSLNode.sunRotation, 0.0]*sqrt(coronaSLNode.displaySize^2 - sunObj.pos.z^2) --z

		--animated sun
		if coronaSLNode.sunRotation.isAnimated == true or coronaSLNode.height.isAnimated == true do
		(
			animate on
			for t in animationRange.start to animationRange.end by 1 do
			(
				at time t
				(
					sunObj.pos = coronaSLNode.pos + coronaSLNode.displaySize*[sin coronaSLNode.height, 0.0, cos coronaSLNode.height]
					sunObj.pos = [coronaSLNode.pos.x , coronaSLNode.pos.y, sunObj.pos.z]  + [cos coronaSLNode.sunRotation , sin coronaSLNode.sunRotation, 0.0]*sqrt(coronaSLNode.displaySize^2 - sunObj.pos.z^2)
				)
			)
		)
	) else (
		sunObj = CreateInstance VRaySun name:(coronaSLNode.name+"_VRaySun") enabled:coronaSLNode.on intensity_multiplier:coronaSLNode.intensity size_multiplier:coronaSLNode.sizeMultiplier invisible:(not coronaSLNode.visibleDirectly)
	)

	if (coronaSLNode.colorMode == 0) do (
		sunObj.color_mode = 1
		sunObj.filter_Color = coronaSLNode.colorDirect
	)

	--excludeList
	if coronaSLNode.excludeList.count != 0 do
	(
		sunObj.inclExclType = 1
		if coronaSLNode.excludeListIncludeMod == false then sunObj.excludeList = coronaSLNode.excludeList else sunObj.includeList = coronaSLNode.excludeList
	)

	if previousVersion then (
		theLayer = layermanager.getLayerFromName coronaSLNode.layer.name
		theLayer.addNode sunObj
		theLayer.addNode targetObj

		delete coronaSLNode
	) else (
		instCount = InstanceMgr.GetInstances coronaSLNode &arrInst
		for i=1 to instCount do (
			arrInst[i].name = (arrInst[i].name+"_VRaySun")
		)
		replaceInstances coronaSLNode sunObj
	)

	return undefined
)

fn normalizeColor col = (
	norm=if (col.r>col.g) then (if (col.r>col.b) then col.r else col.b) else (if (col.g>col.b) then col.g else col.b)
	result=color ((col.r*255.0)/norm) ((col.g*255.0)/norm) ((col.b*255.0)/norm)
	result
)

fn replaceLightNodeVRIESLight node = (
	local newLight=CreateInstance VRayIES name:(node.name+"_VRayIESLight")

	if (newLight!=undefined) do (
		newLight.targeted=(node.target!=undefined)
		newLight.enabled=node.on
		newLight.color_mode=0
		newLight.color=normalizeColor node.color
		newLight.rotation_X=node.xRotation
		newLight.rotation_Y=node.yRotation
		newLight.rotation_Z=node.zRotation
		newLight.use_light_shape=1
		if (node.webFile!=undefined) then (
			newLight.ies_file=node.webFile
			local powerCoeff=node.intensity/node.originalIntensity
			newLight.power=newLight.power*powerCoeff
		) else (
			newLight.power=node.intensity*4*pi
		)
		if (node.useMultiplier) do (newLight.power=(newLight.power*node.multiplier)/100.0)
		if (hasProperty newLight "override_shape") then (
			newLight.override_shape=true
			case of (
				(node.type==#Target_Point or node.type==#Free_Point): (
					newLight.shape=0
				)

				(node.type==#Target_Line or node.type==#Free_Line or
				node.type==#Target_Linear or node.type==#Free_Linear): (
					newLight.shape=1
					newLight.length=node.light_length
				)

				(node.type==#Target_Rectangle or node.type==#Free_Rectangle or
				node.type==#Target_Area or node.type==#Free_Area): (
					newLight.shape=1
					newLight.width=node.light_width
					newLight.length=node.light_length
				)

				(node.type==#Target_Disc or node.type==#Free_Disc): (
					newLight.shape=2
					newLight.diameter=node.light_radius*2
				)

				(node.type==#Target_Sphere or node.type==#Free_Sphere): (
					newLight.shape=3
					newLight.diameter=node.light_radius*2
				)

				(node.type==#Target_Cylinder or node.type==#Free_Cylinder): (
					newLight.shape=5
					newLight.length=node.light_length
					newLight.diameter=node.light_radius*2
				)
			)
		)
		if (node.includeList!=undefined) do (newLight.includeList=node.includeList)
		if (node.excludeList!=undefined) do (newLight.excludeList=node.excludeList)
	)

	instCount = InstanceMgr.GetInstances node &arrInst
	for i=1 to instCount do (
		arrInst[i].name = (arrInst[i].name+"_VRayIESLight")
	)
	replaceInstances node newLight
	newLight
)

fn replaceLightNodeVRayLight node = (
	local newLight=CreateInstance VRayLight name:(node.name+"_VRayLight")

	if (newLight!=undefined) do (
		newLight.targeted=(node.target!=undefined)
		if newLight.targeted then newLight.target_distance = node.targetDistance
		newLight.on=node.on
		newLight.color_mode=0
		newLight.color=normalizeColor node.color
		newLight.normalizeColor = 1
		newLight.multiplier=node.intensity*4*pi
		newLight.invisible= not node.baseObject.Area_Light_Sampling_Custom_Attribute.mr_EnableLightShapeRendering
		newLight.texmap_on=node.projector
		if (node.projector) do newLight.texmap=node.projectorMap
		if (node.useMultiplier) do (newLight.multiplier=(newLight.multiplier*node.multiplier)/100.0)
		case of (
			(node.type==#Target_Rectangle or node.type==#Free_Rectangle or
			node.type==#Target_Area or node.type==#Free_Area): (
				newLight.type=0
				newLight.size0=node.light_width / 2.0
				newLight.size1=node.light_length / 2.0
				newLight.DoubleSided = true
			)

			(node.type==#Target_Disc or node.type==#Free_Disc): (
				newLight.type=4
				newLight.size0=node.light_radius
				newLight.DoubleSided = true
			)

			(node.type==#Target_Sphere or node.type==#Free_Sphere): (
				newLight.type=2
				newLight.size0=node.light_radius
			)
		)

		if (node.includeList!=undefined) do (newLight.includeList=node.includeList)
		if (node.excludeList!=undefined) do (newLight.excludeList=node.excludeList)
	)

	instCount = InstanceMgr.GetInstances node &arrInst
	for i=1 to instCount do
		arrInst[i].name = (arrInst[i].name+"_VRayLight")
	replaceInstances node newLight
	newLight
)

fn replaceLight_mr_Sun lightsys = (
	if (classof lightsys.sun==mr_Sun) do (
		local mrSun_on=lightsys.sun.on
		local mrSun_skymult=lightsys.sun.skymult

		lightsys.sun=createInstance VRaySun
		lightsys.sun.enabled=mrSun_on
		lightsys.sun.intensity_multiplier=mrSun_skymult

		lightsys.sky=undefined
	)
)

fn replaceLightNode node = (
	local lightClassStandard=#(Free_Light, Target_Light,
									Free_Point, Target_Point,
									Free_Linear, Target_Linear,
									Free_Area, Target_Area,
									Free_Disc, Target_Disc,
									Free_Sphere, Target_Sphere,
									Free_Cylinder, Target_Cylinder)

	local lightClassCorona=#(CoronaLight, CoronaSun)
	local newLight
	if (isValidNode node) do (
		if ((findItem lightClassStandard (classof node))!=0) do (
			case (node.distribution) of (
				0: newLight=replaceLightNodeVRayLight node	-- Uniform Spherical
				1: newLight=undefined						-- Spotlight
				2: newLight=replaceLightNodeVRayLight node	-- Uniform Diffuse
				3: newLight=replaceLightNodeVRIESLight node	-- Photometric Web
			)
		)

		local foundCL = 0
		if ((foundCL = findItem lightClassCorona (classof node))!=0) do (
			case (foundCL) of (
				1: newLight=replaceCoronaLightNodeVRayLight node	-- Uniform Spherical
				2: newLight=replaceCoronaSunLightNodeVRayLight node
			)
		)
	)

	if (newLight!=undefined) do (
		print (node.name)
	)

	newLight!=undefined
)

fn lightsToVRay selectedOnly = (
	format "Converting scene Lights...\n"
	local oldLights=#()

	-- First traverse scene for daylight systems otherwise the sun/sky lights in them will appear as standalone
	-- lights to the normal lights replace traversal.
	if (vraySceneConverterRollout.check_optLights.checked) do (
		for i=1 to $helpers.count do (
			if (classof $helpers[i]==DaylightAssemblyHead) do (
				replaceLight_mr_Sun $helpers[i]
			)
		)

		if ((environmentMap!=undefined) and (classof environmentMap==mr_Physical_Sky)) do (
			environmentMap=VRaySky()
		)
	)

	if selectedOnly do  format "Converting scene Lights for selected lights...\n"
	for i=1 to $lights.count do (
		if selectedOnly then (
			if $lights[i].isSelected do
				if (replaceLightNode $lights[i]) do ( append oldLights $lights[i] )
		) else (
			if (replaceLightNode $lights[i]) do ( append oldLights $lights[i] )
		)
	)
	-- old lights need to stay until the conversion is done, because deleteing them
	-- on the run messes nodes' order during traversal
	format "Done\n"
)

fn replaceLightMeter node=(
	local vrLightMeter = undefined
	if (classof node==LightMeter) do (
		vrLightMeter = CreateInstance VRayLightMeter
		-- try to convert
		format "  %\n" node
		vrLightMeter.active = node.active
		vrLightMeter.width = node.width
		vrLightMeter.length = node.length
		vrLightMeter.wsegs = node.width_segs
		vrLightMeter.lsegs = node.length_segs
		vrLightMeter.orient = node.showOrientation
		vrLightMeter.dispvalue = node.valuesToDisplay
		vrLightMeter.text = node.showViewportText
		vrLightMeter.colors = node.showPsuedoColor

		vrLightMeter.log = (PseudoColorManager.scaleType == #logarithmic)
		vrLightMeter.min_lux = PseudoColorManager.rangeStart
		vrLightMeter.max_lux = PseudoColorManager.rangeEnd

		instCount = InstanceMgr.GetInstances node &arrInst
		for i=1 to instCount do
			arrInst[i].name = (arrInst[i].name+"_VRayLightMeter")

		replaceInstances node vrLightMeter
	)

	if (vrLightMeter != undefined) do (
		print (node.name)
	)

	vrLightMeter != undefined
)

fn lightmetersToVRay selectedOnly = (
	format "Converting scene LightMeters...\n"
	local oldLightMeters=#()
	if selectedOnly do  format "Converting scene LightMeters for selected ones...\n"
	for i=1 to $helpers.count do (
		if selectedOnly then (
			if $helpers[i].isSelected do
				if (replaceLightMeter $helpers[i]) do ( append oldLightMeters $helpers[i] )
		) else (
			if (replaceLightMeter $helpers[i]) do ( append oldLightMeters $helpers[i] )
		)
	)

	format "Done\n"
)

fn replaceCameraNode node = (
	local cameraClassStandard=#(Freecamera, Targetcamera)

	local newCamera
	local newCameraName
	if (isValidNode node) do (
		local camIdx = findItem cameraClassStandard (classof node)
		if (camIdx!=0) do (
			--Implementation

			if (vraySceneConverterRollout.ddListCameras.selection == 3) then (
				newCamera=CreateInstance Physical specify_fov: on
				newCameraName = "_PhysicalCamera"
			)

			if (vraySceneConverterRollout.ddListCameras.selection == 2) then (
				newCamera=CreateInstance VRayPhysicalCamera specify_fov: on
				newCameraName = "_VRayPhysicalCamera"
			)

			if (newCamera!=undefined) do (
				local newCameraClass = classof newCamera
				if (camIdx == 1) then (
					newCamera.targeted = off
					newCamera.target_distance = node.baseObject.targetDistance
				)
				else (
					newCamera.targeted = off
					newCamera.target_distance = node.targetDistance
					newCamera.targeted = on
				)

				if (node.fov.isAnimated) then(
					classOfNodeController = classof node.fov.controller
					--newCamera.fov.controller=bezier_float()
					newCamera.fov.controller = createInstance classOfNodeController
					nkeys = node.fov.keys
					for k in nkeys do appendKey newCamera.fov.keys k
					for i=1 to nkeys.count do (
						newCamera.fov.keys[i].value=node.fov.keys[i].value
					)
				) else (
					newCamera.fov = node.fov -- By some reason Max Physical Camera does not accept fov value
				)

				newCamera.specify_fov = off

				if (node.showRanges == on) then (
					newCamera.environment_near = node.nearrange
					newCamera.environment_far = node.farrange
				)

				newCamera.clip_on = node.clipManually
				newCamera.clip_near = node.nearclip
				newCamera.clip_far = node.farclip

				if (node.mpassEnabled == on) then (
					local mpEffectClass = classof node.mpassEffect
					case mpEffectClass of (
						Depth_of_FieldMPassCamEffect: (
							newCamera.use_dof = on
							if (newCameraClass == Physical) then (
								newCamera.motion_blur_enabled = off
								newCamera.focus_distance = node.mpassEffect.focalDepth
								if (node.mpassEffect.useTargetDistance == on) then newCamera.specify_focus = 0
								else newCamera.specify_focus = 1
							)
							else newCamera.use_moblur = off
						)
						Motion_BlurMPassCamEffect: (
							newCamera.use_dof = off
							if (newCameraClass == Physical) do newCamera.motion_blur_enabled = on
							if (newCameraClass == VRayPhysicalCamera) do newCamera.use_moblur = on
						)
						Depth_of_Field__mental_ray: (
							newCamera.use_dof = on
						)
					)
				)

				instCount = InstanceMgr.GetInstances node &arrInst
				for i=1 to instCount do
					arrInst[i].name = (arrInst[i].name+newCameraName)
				replaceInstances node newCamera
			)
		)
	)

	if (newCamera!=undefined) do (
		print (node.name)
	)

	newCamera!=undefined
)

fn camerasToVRay selectedOnly = (
	if (vraySceneConverterRollout.ddListCameras.selection > 1) do (
		format "Converting scene Cameras...\n"
		local oldCameras=#()

		if selectedOnly do format "Converting scene Cameras for selected cameras only...\n"
		for i=1 to $cameras.count do (
			if selectedOnly then (
				if $cameras[i].isSelected do
					if (replaceCameraNode $cameras[i]) do ( append oldCameras $cameras[i] )
			) else (
				if (replaceCameraNode $cameras[i]) do ( append oldCameras $cameras[i] )
			)
		)
		-- old cameras need to stay until the conversion is done, because deleteing them
		-- on the run messes nodes' order during traversal
		format "Done\n"
	)
)

--Convert Corona Proxy to VRay Proxy
fn getOrigCoronaProxy  selectedOnly =  ( --get list of original proxies
	arrayOrigCoronaProxy = #()
	arrayOrigCoronaProxy_pathes = #()
	 for i in objects where (classOf i == CProxy)  and (not selectedOnly or selectedOnly and i.isSelected) do
	 (
		 if (findItem arrayOrigCoronaProxy_pathes  i.filename) == 0 and doesFileExist i.filename != false do
		 (
			append arrayOrigCoronaProxy i
			append arrayOrigCoronaProxy_pathes i.filename
		)
	)
	arrayOrigCoronaProxy
)

fn getData_InstProxy_byOrig ocp = ( --get list of dependent proxies
	arrayCoronaProxy = #()
	for i in objects where classOf i == CProxy do
	(
		if i.filename != undefined and i.filename == ocp.filename do append arrayCoronaProxy #(i, i.name, getFilenameFile  i.filename, getFilenamePath i.filename, i.transform, i.previzType)
	)
	arrayCoronaProxy
)

fn coronaProxiesToVRay thePath selectedOnly = (
	OrigCorPr = getOrigCoronaProxy selectedOnly --list of corona proxies
	if OrigCorPr.count != 0 do (
		numCP = OrigCorPr.count
		for i = 1 to numCP do (
			theData_InstCorPr = getData_InstProxy_byOrig OrigCorPr[i] -- list of proxies related to each corona poxy

			tempObj = instance OrigCorPr[i] wirecolor:red
			tempObj.transform = (matrix3 [1,0,0] [0,1,0] [0,0,1] [0,0,0])
			tempObj.previzType = 3
			converttomesh tempObj
			objNF = (tempObj.numfaces/10.0) as integer -- preview 1/10 ot numfaces


			tempObj.name = theData_InstCorPr[1][3]
			select tempObj
			if thePath != undefined and thePath != "Select path please..." then (
				tempVRayPr = vrayMeshExport tempObj autoCreateProxies:true maxPreviewFaces:objNF  meshfile:(thePath + "\\" + theData_InstCorPr[1][3])
			) else (
				tempVRayPr = vrayMeshExport tempObj autoCreateProxies:true maxPreviewFaces:objNF  meshfile:(theData_InstCorPr[1][4] + theData_InstCorPr[1][3])
			)
			tempVRayPr[1].display = 0
			tempVRayPr[1].visibility_lists_type = 0
			tempVRayPr[1].visibility_list_names[1] = false
			for i in (getData_InstProxy_byOrig OrigCorPr[i]) do i[1].name = uniqueName tempObj.name

			--replace each corona proxy with vray proxy -> tempVRayPr
			instanceReplace OrigCorPr[i] tempVRayPr[1]
			delete tempVRayPr
		)
	)
)

fn lightingAnalysisOverlayToVRayLightingAnalysis = (
	theManager = maxOps.GetRenderElementMgr #Production
	listOfLightingEffectsIdxs = #()
	for i=1 to numEffects do (
		iEffect = (getEffect i)
		if classof iEffect == Lighting_Analysis_Overlay then (
			local newVRLightingAnalysis = VRayLightingAnalysis name: iEffect.name
			newVRLightingAnalysis.vrayVFB = (IsActive iEffect)
			if iEffect.quantity == 0 then newVRLightingAnalysis.quantity = 1
			else newVRLightingAnalysis.quantity = 0
			newVRLightingAnalysis.scale = iEffect.scaleType
			newVRLightingAnalysis.min_value = iEffect.rangeStart
			newVRLightingAnalysis.max_value = iEffect.rangeEnd
			if iEffect.shownumbers == on then (
				newVRLightingAnalysis.display = 1
				newVRLightingAnalysis.grid_horizontal_spacing = iEffect.horizontalspacing
				newVRLightingAnalysis.grid_vertical_spacing = iEffect.verticalspacing
				newVRLightingAnalysis.fade_background_image = iEffect.fadeBackground
			)
			theManager.AddRenderElement  newVRLightingAnalysis
			append listOfLightingEffectsIdxs i
		)
	)
	print listOfLightingEffectsIdxs
	for idx = listOfLightingEffectsIdxs.count to 1 by -1 do (
		deleteEffect listOfLightingEffectsIdxs[idx]
	)
)

fn ConvertToVRay selectedOnly = (
	mtlsToVRay selectedOnly
	shadowsToVRay selectedOnly
	lightmetersToVRay selectedOnly
	lightsToVRay selectedOnly
	camerasToVRay selectedOnly
	coronaProxiesToVRay undefined selectedOnly
	lightingAnalysisOverlayToVRayLightingAnalysis()
)
convertToVRay=ConvertToVRay

-- Bitmap2VRayHDRI converter
convertB2HDRI_ConvertEXR=false
convertB2HDRI_ConverterErrors=0
convertB2HDRI_ErrorsInTXExisting=0 --named only for Tx because of upcoming exr conversion elimination

listOfSuppFormats = #(".png", ".bmp", ".tga", ".hdr", ".sgi", ".rgb", ".rgba", ".jpg", ".jpeg", ".jpe", ".exr", ".pic", ".tif", ".tiff", ".psd")
mapfiles=#() --list of bitmap image file paths

--The next function is passed as 'enumeratefiles' argument and is called for each file found.
--The mapfile argument is the string file name supplied by the enumerator. 
fn addmap mapfile = (
	local index=finditem mapfiles mapfile
	local idFormatType = finditem listOfSuppFormats (tolower(getFilenameType mapfile))
	if (index==0 and idFormatType > 0) do append mapfiles mapfile
)

fn convertImgToTiledEXR convertHalf convertCompression convertTilesize convertLinear = (
	--print "********** START CONVERSION  Image --> Tiled EXR ****************"
	start = timeStamp()
	if (convertPath==undefined) do (
		local ver=getMaxReleaseAsString()
		convertPath = getVRayInstallPath ver
	)
	mapfiles=#()
	--The following method is used to collect a list of the bitmap files used in a scene.
	enumeratefiles addmap
	local iImg = 0
	local mapfilesCount = mapfiles.count
	--The real conversion done by 'img2tiledexr.exe' according to specified options for each mapfile from list - mapfiles
	for mapfile in mapfiles do (
		windows.processPostedMessages()
		fullImgPath = mapPaths.getFullFilePath mapfile
		if ((classof fullImgPath)==String) then (
			if ((stricmp (getFilenameType fullImgPath) ".exr")!=0) do (
				outpath=(getFilenamePath fullImgPath)+(getFilenameFile fullImgPath)+".exr"
				if (not (doesFileExist outpath)) do (
					if ((classof convertPath)==String) do (
						local cmdConverter=("\""+convertPath+"\" \""+fullImgPath+"\" \""+outpath+"\"")
						if (convertHalf) then (append cmdConverter " -half")
						else (append cmdConverter " -32bit")
						append cmdConverter (" -compression "+convertCompression)
						append cmdConverter (" -tileSize "+(convertTilesize as String))
						append cmdConverter (" -linear "+convertLinear)

						vrayShellLaunch cmdConverter waitExec:true hideConsole:true
					)
				)

				if (not (doesFileExist outpath)) do (
						format "Failed to convert % to tiled EXR,\n please check if you have write permissions for %\n" fullImgPath outpath
					)
				)
			)
		iImg = iImg + 1
		convertB2HDRI_Rollout.prog_convert.value = 100.*iImg/mapfilesCount
	)
	convertB2HDRI_Rollout.prog_convert.value = 0
	end = timeStamp()
	format "Converting took % seconds\n" ((end - start) / 1000.0)
	--print "**********   END CONVERSION  Image --> Tiled EXR ****************"
)

fn convertImgToTX = (
	--print "********** START CONVERSION  Image --> Tiled, MIP-mapped textures ****************"
	start = timeStamp()
	if (convertPath==undefined) do (
		local ver=getMaxReleaseAsString()
		convertPath = getVRayInstallPath ver
	)
	mapfiles=#()
	--The following method is used to collect a list of the bitmap files used in a scene.
	enumeratefiles addmap
	local iImg = 0
	local mapfilesCount = mapfiles.count
	--The real conversion done by 'maketx.exe' for each mapfile from list - mapfiles
	for mapfile in mapfiles do (
		windows.processPostedMessages()
		fullImgPath = mapPaths.getFullFilePath mapfile
		if ((classof fullImgPath)==String) then (
			if ((stricmp (getFilenameType fullImgPath) ".tx")!=0) do (
				outpath=(getFilenamePath fullImgPath)+(getFilenameFile fullImgPath)+".tx"
				if (not (doesFileExist outpath)) do (
					if ((classof convertPath)==String) do (
						local cmdConverter=("\""+convertPath+"\" \""+fullImgPath+"\" -o \""+outpath+"\" --tile 64 64")
						vrayShellLaunch cmdConverter waitExec:true hideConsole:true
					)
				)

				if (not (doesFileExist outpath)) do (
					format "Failed to convert % to TX,\n please check if you have write permissions for %\n" fullImgPath outpath
				)
			)
		)
		iImg = iImg + 1
		convertB2HDRI_Rollout.prog_convert.value = 100.*iImg/mapfilesCount
	)
	convertB2HDRI_Rollout.prog_convert.value = 0
	end = timeStamp()
	format "Converting took % seconds\n" ((end - start) / 1000.0)
	--print "**********   END CONVERSION  Image --> Tiled, MIP-mapped textures ****************"
)



rollout ConvertB2HDRI_TEXRConverterRollout "Img2tiledexr tool conversion options" width: 460 height: 200 (
	label label_message "If the img2tiledexr tool was not found it should be set manually." width:400 height:13 align:#center offset:[0, 10]

	edittext edit_path width:394 across:2 align:#left offset:[13, 0]
	button btn_path "..." width:20 height:18 align:#right offset:[0, 0]

	checkbox check_half "Half - store data as 16-bit floating point numbers." checked:true width:400 align:#center offset:[0, 10]

	label label_compression "Compression method:" width:105 height:13 across:2 align:#left offset:[18, 2]
	dropdownlist drop_compression items:#("None", "RLE", "Zips", "Zip", "PIZ", "PXR24") selection:4 width:100 align:#center offset:[-150, 0]

	label label_tilesize "Tile size:" width:40 height:13 across:2 align:#left offset:[18, 2]
	dropdownlist drop_tilesize items:#("32x32", "64x64", "128x128") selection:2 width:100 align:#center offset:[-150, 0]

	label label_linearmode "Linear mode:" width:60 height:13 across:2 align:#left offset:[18, 2]
	dropdownlist drop_linearmode items:#("Off", "On", "Auto") selection:3 width:100 align:#center offset:[-150, 0]

	button btn_set "OK" width:60 align:#right across:2 offset:[150, 0]
	button btn_cancel "Cancel" width:60 align:#right

	on ConvertB2HDRI_TEXRConverterRollout open do (
		if (convertB2HDRI_Rollout.check_convertEXR.checked) do (
			local tmpPath=convertB2HDRI_Rollout.convertPath
			if (tmpPath!=undefined) do (
				if (doesFileExist tmpPath) do (
					edit_path.text=tmpPath
					edit_path.enabled=false
				)
			)
		)
	)

	on btn_path pressed do (
		local toolPath=getOpenFileName filename:edit_path.text \
								types:"Executable File (img2tiledexr.exe)|img2tiledexr.exe"
		if (toolPath!=undefined) do (
			edit_path.text=toolPath
		)
	)

	on btn_set pressed do (
		convertB2HDRI_Rollout.convertPath=edit_path.text
		convertB2HDRI_Rollout.convertHalf=check_half.checked
		convertB2HDRI_Rollout.convertCompression=case drop_compression.selection of (
			1:	"none"
			2:	"rle"
			3:	"zips"
			4:	"zip"
			5:	"piz"
			6:	"pxr24"
		)
		convertB2HDRI_Rollout.convertTilesize=case drop_tilesize.selection of (
			1:	32
			2:	64
			3:	128
		)
		convertB2HDRI_Rollout.convertLinear=case drop_linearmode.selection of (
			1:	"off"
			2:	"on"
			3:	"auto"
		)

		destroyDialog ConvertB2HDRI_TEXRConverterRollout
	)

	on btn_cancel pressed do (
		convertB2HDRI_Rollout.cancelled=true
		destroyDialog ConvertB2HDRI_TEXRConverterRollout
	)
)

rollout ConvertB2HDRI_MakeTXConverterRollout "MakeTX tool conversion options" width: 460 height: 80 (
	label label_message "If the maketx tool was not found it should be set manually." width:400 height:13 align:#center offset:[0, 10]

	edittext edit_path width:394 across:2 align:#left offset:[13, 0]
	button btn_path "..." width:20 height:18 align:#right offset:[0, 0]

	button btn_set "OK" width:60 align:#right across:2 offset:[150, 0]
	button btn_cancel "Cancel" width:60 align:#right

	on ConvertB2HDRI_MakeTXConverterRollout open do (
		if (convertB2HDRI_Rollout.check_convertEXR.checked) do (
			local tmpPath=convertB2HDRI_Rollout.convertPath
			if (tmpPath!=undefined) do (
				if (doesFileExist tmpPath) do (
					edit_path.text=tmpPath
					edit_path.enabled=false
				)
			)
		)
	)

	on btn_path pressed do (
		local toolPath=getOpenFileName filename:edit_path.text \
								types:"Executable File (maketx.exe)|maketx.exe"
		if (toolPath!=undefined) do (
			edit_path.text=toolPath
		)
	)

	on btn_set pressed do (
		convertB2HDRI_Rollout.convertPath=edit_path.text
		destroyDialog ConvertB2HDRI_MakeTXConverterRollout
	)

	on btn_cancel pressed do (
		convertB2HDRI_Rollout.cancelled=true
		destroyDialog ConvertB2HDRI_MakeTXConverterRollout
	)
)

--Just check if there is a valid converted to EXR file and return it.
--If EXR file couldn't be found in the same place like the original file then returns the original one .
fn convertB2HDRI_img2tiledexr imgpath=(
	local fullImgPath = mapPaths.getFullFilePath imgpath
	if ((classof fullImgPath)==String and fullImgPath != "") then (
		if ((stricmp (getFilenameType fullImgPath) ".exr")!=0) then (
			outpath=(getFilenamePath fullImgPath)+(getFilenameFile fullImgPath)+".exr"
		)
		else (
			outpath=fullImgPath
		)

		if (doesFileExist outpath) then (
			outpath
		)
		else (
			if (convertB2HDRI_ConverterErrors!=undefined) do (
				convertB2HDRI_ConverterErrors+=1
			)
			convertB2HDRI_ErrorsInTXExisting+=1
			format "Image % does not exist.\n" outpath
			fullImgPath
		)
	)
	else (
		if (fullImgPath=="") do (
			if (convertB2HDRI_ConverterErrors!=undefined) do (
				convertB2HDRI_ConverterErrors+=1
			)
			convertB2HDRI_ErrorsInTXExisting+=1
			format "Image % does not exist.\n" imgpath
		)
		imgpath
	)
)

--Just check if there is a valid converted to TX file and return it.
--If TX file couldn't be found in the same place like the original file then returns the original one .
fn convertB2HDRI_img2TX imgpath=(
	local fullImgPath = mapPaths.getFullFilePath imgpath
	if ((classof fullImgPath)==String and fullImgPath != "") then (
		if ((stricmp (getFilenameType fullImgPath) ".tx")!=0) then (
			outpath=(getFilenamePath fullImgPath)+(getFilenameFile fullImgPath)+".tx"
		)
		else (
			outpath=fullImgPath
		)

		if (doesFileExist outpath) then (
			outpath
		)
		else (
			if (convertB2HDRI_ConverterErrors!=undefined) do (
				convertB2HDRI_ConverterErrors+=1
			)
			convertB2HDRI_ErrorsInTXExisting+=1
			format "Image % does not exist.\n" outpath
			fullImgPath
		)
	)
	else (
		if (fullImgPath=="") do (
			if (convertB2HDRI_ConverterErrors!=undefined) do (
				convertB2HDRI_ConverterErrors+=1
			)
			convertB2HDRI_ErrorsInTXExisting+=1
			format "Image % does not exist.\n" imgpath
		)
		imgpath
	)
)

fn convertB2HDRI_Texmap texmap=(
	if (texmap!=undefined and (classof texmap)==Bitmaptexture) then (
		local tex=undefined
		local mapFileNameIsValid=(texmap.fileName != undefined and texmap.fileName != "")
		--tex.gamma=1/2.2
		--tex.color_space=3 --Lele: set VRayHDRI's Color space to "From Max"

		convertB2HDRI_Rollout.replaceImgCounter = convertB2HDRI_Rollout.replaceImgCounter + 1
		convertB2HDRI_Rollout.prog_convert.value = 100.*convertB2HDRI_Rollout.replaceImgCounter/convertB2HDRI_Rollout.replaceImgNums

		if (convertB2HDRI_ConvertEXR) then (
			--Here we would like to create VRayHDRI texture with Bitmaptexture file converted to EXR/TX bitmap file.
			convertB2HDRI_ErrorsInTXExisting=0
			local convertB2HDRI_ConvertLinear=convertB2HDRI_Rollout.convertLinear

			local resultHDRIMapName=undefined
			if useMakeTx then
				resultHDRIMapName=if(mapFileNameIsValid) then convertB2HDRI_img2TX texmap.fileName else undefined
			else
				resultHDRIMapName=if(mapFileNameIsValid) then convertB2HDRI_img2tiledexr texmap.fileName else undefined

			--If there isn't successful file conversion to EXR/TX or Bitmaptexture file name is not valid,
			--the original Bitmaptexture won't be replaced with VRayHDRI
			if (convertB2HDRI_ErrorsInTXExisting==0 and resultHDRIMapName!=undefined) do (
				tex=VRayHDRI()
				tex.name=texmap.name+"_VRay"
				tex.HDRIMapName=resultHDRIMapName
				if (tex.HDRIMapName!=undefined) do (
					local gamma=vrayGetBitmapGamma texmap
					if (gamma!=undefined) then (
						-- If we used img2tiledexr for conversion, we need to correct the gamma specified in the Bitmap texture with
						-- the gamma that was applied when converting the image.
						if not useMakeTx then (
							-- Determine the gamma correction that was applied to the file when converting it
							local savedGamma=1.0
							if convertB2HDRI_ConvertLinear=="on" then savedGamma=2.2 -- We have applied 2.2 gamma when converting
							else if convertB2HDRI_ConvertLinear=="off" then savedGamma=1.0 -- We have not applied any gamma when converting
							else (
								-- Auto mode - we have applied 1.0 gamma for .exr/.hdr images and 2.2 gamma for everything else
								local isHDRI=(((stricmp (getFilenameType texmap.fileName) ".exr")==0) OR ((stricmp (getFilenameType texmap.fileName) ".hdr")==0))
								if isHDRI then savedGamma=1.0
								else savedGamma=2.2
							)

							-- Correct the gamma specified in the bitmap with the one that was applied during conversion.
							gamma=gamma/savedGamma
						)
						tex.gamma=1.0/gamma
					)
				)
			)
		)
		else (
			--Here we would like to create VRayHDRI texture without Bitmaptexture file convertion.
			tex=VRayHDRI()
			tex.name=texmap.name+"_VRay"
			if (mapFileNameIsValid) then (
				tex.HDRIMapName=texmap.fileName
			) else (
				format "Image path for % does not exist.\n" texmap
			)

			local gamma=vrayGetBitmapGamma texmap
			if (gamma!=undefined) then (
				tex.gamma=1.0/gamma
			)
		)


		if (tex!=undefined) then (
			tex.mapType=4
			tex.UVGen.blur=texmap.coords.blur
			tex.UVGen.mapping=texmap.coords.mapping
			tex.UVGen.mapChannel=texmap.coords.mapChannel
			tex.UVGen.mappingType=texmap.coords.mappingType
			tex.UVGen.UVW_Type=texmap.coords.UVW_Type
			tex.UVGen.U_Mirror=texmap.coords.U_Mirror
			tex.UVGen.V_Mirror=texmap.coords.V_Mirror
			tex.UVGen.U_Tile=texmap.coords.U_Tile
			tex.UVGen.V_Tile=texmap.coords.V_Tile
			tex.UVGen.showMapOnBack=texmap.coords.showMapOnBack
			tex.UVGen.Noise_On=texmap.coords.Noise_On
			tex.UVGen.Noise_Animate=texmap.coords.Noise_Animate
			tex.UVGen.realWorldScale=texmap.coords.realWorldScale
			tex.UVGen.realWorldHeight=texmap.coords.realWorldHeight
			tex.UVGen.realWorldWidth=texmap.coords.realWorldWidth
			tex.UVGen.phase=texmap.coords.phase
			tex.UVGen.U_Angle=texmap.coords.U_Angle
			tex.UVGen.V_Angle=texmap.coords.V_Angle
			tex.UVGen.W_Angle=texmap.coords.W_Angle
			tex.UVGen.Noise_Amount=texmap.coords.Noise_Amount
			tex.UVGen.Noise_Size=texmap.coords.Noise_Size
			tex.UVGen.Noise_Levels=texmap.coords.Noise_Levels
			tex.UVGen.Blur_Offset=texmap.coords.Blur_Offset
			tex.UVGen.u_tiling=texmap.coords.u_tiling
			tex.UVGen.v_tiling=texmap.coords.v_tiling
			tex.UVGen.u_offset=texmap.coords.u_offset
			tex.UVGen.v_offset=texmap.coords.v_offset

			--Convert crop/placement
			tex.cropplace_on = texmap.apply
			tex.cropplace_mode = texmap.cropPlace
			tex.cropplace_u = texmap.clipu
			tex.cropplace_v = texmap.clipv
			tex.cropplace_width = texmap.clipw
			tex.cropplace_height = texmap.cliph

			--Convert output parameters including curve control
			replaceInstances tex.output (copy texmap.output)

			--Frees up all the memory used by the image file bitmap caches.
			--This is useful if memory is fragmented with a lot of different bitmaps and you want to have just the ones currently active reloaded.
			freescenebitmaps()
			--invokes the garbage collector without flush the undo system
			gc lite:true
		)
		tex
	)
	else (
		undefined
	)
)

fn convertB2HDRI_SubTex texmap = (
	if (texmap!=undefined) then (
		-- check if we have already converted this texture; if yes, just return
		-- the already converted one
		local i=findItem orig_bitmaptextures texmap
		if i>0 then return new_hdritexmaps[i]

		for i = 1 to (getNumSubTexmaps texmap) do (
			local tex=(convertB2HDRI_SubTex (getSubTexmap texmap i))
			if (tex!=undefined) do (
				setSubTexmap texmap i tex
			)
		)

		local new_tex = texmap
		if ((classof texmap)==Bitmaptexture) then (
			new_tex=convertB2HDRI_Texmap texmap
		)

		-- if converted, add to the list of converted textures
		if new_tex!=texmap then (
			append orig_bitmaptextures texmap
			append new_hdritexmaps new_tex
			return new_tex
		)
	)
	return undefined
)

fn convertB2HDRI_SubMtl material = (
	if (material!=undefined) do (
		for i=1 to (getNumSubMtls material) do (
			convertB2HDRI_SubMtl (getSubMtl material i)
		)

		for i=1 to (getNumSubTexmaps material) do (
			local tex=(convertB2HDRI_SubTex (getSubTexmap material i))
			if (tex!=undefined) do (setSubTexmap material i tex)
		)
	)
)

fn convertB2HDRI_ArrayParameter propArray = (
	if ((classof propArray)==ArrayParameter) then (
		for i=1 to propArray.count do (
			if (propArray[i]!=undefined) do (
				if ((superclassof propArray[i])==textureMap) then (
					hdriTex=convertB2HDRI_SubTex propArray[i]
					if (hdriTex!=undefined) do propArray[i]=hdriTex
				) else if((superclassof propArray[i])==material) then (
					convertB2HDRI_SubMtl propArray[i]
				) else convertB2HDRI_ArrayParameter propArray[i]
			)
		)
	)
)

fn convertB2HDRI_Objects obj = (
	if(obj != undefined) do (
		local specificObj
		local classOfObj = classof obj
		if ((superclassof obj)==light) then (
			if (classOfObj == miAreaLightOmni) then (
				specificObj = obj.Omni_Light
			)
			else if (classOfObj == miAreaLight) then (
				specificObj = obj.Target_Spot
			)
			else (
				specificObj = obj
			)
		)
		else (
			specificObj = obj
		)

		local listPropNames = getPropNames specificObj

		local hdriTex = undefined
		local propValue = undefined

		for propName in listPropNames do (
			if (isProperty specificObj propName) then (
				propValue = getProperty specificObj propName
				if ((superclassof propValue)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex propValue
					if (hdriTex!=undefined) do setProperty specificObj propName hdriTex
				) else if((superclassof propValue)==material) then (
					convertB2HDRI_SubMtl propValue
				) else convertB2HDRI_ArrayParameter propValue
			)
		)
	)
)

fn convertB2HDRI_Node node applyOnObjects = (
	if isValidNode node then (
		convertB2HDRI_SubMtl node.material
		if (applyOnObjects) do (convertB2HDRI_Objects node.baseObject)
		--convert bitmaps of node modifiers
		for imod in node.modifiers do (
			convertB2HDRI_Objects imod
		)
	)
)

fn convertB2HDRI_RenderSettings = (
	if(checkProductionRenderer()) then (
		productRenderer = renderers.production
		listRenderSettingsNames = getPropNames productRenderer
		for rParamName in listRenderSettingsNames do (
			if (isProperty productRenderer rParamName) then (
				rParam = getProperty productRenderer rParamName
				if ((superclassof rParam)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex rParam
					if (hdriTex!=undefined) do setProperty productRenderer rParamName hdriTex
				) else if ((superclassof rParam)==material) then (
					convertB2HDRI_SubMtl rParam
				) else convertB2HDRI_ArrayParameter rParam
			)
		)
	)
)

fn convertB2HDRI_Environment = (
	-- Convert Environment map
	local envtex=convertB2HDRI_SubTex environmentMap
	if (envtex!=undefined) do environmentMap=envtex

	-- Convert Atmosphere effects
	for iAtmos=1 to numAtmospherics do (
		currAtmospheric = getAtmospheric iAtmos
		listAtmosNames = getPropNames currAtmospheric

		for atmosName in listAtmosNames do (
			if (isProperty currAtmospheric atmosName) then (
				atmosParam = getProperty currAtmospheric atmosName
				if ((superclassof atmosParam)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex atmosParam
					if (hdriTex!=undefined) do setProperty currAtmospheric atmosName hdriTex
				) else if ((superclassof atmosParam)==material) then (
					convertB2HDRI_SubMtl atmosParam
				) else convertB2HDRI_ArrayParameter atmosParam
			)
		)
	)

	-- Convert Environment effects
	for iEffect=1 to numEffects do (
		currEffect = getEffect iEffect
		listEffectNames = getPropNames currEffect
		for effectName in listEffectNames do (
			if (isProperty currEffect effectName) then (
				effectParam = getProperty currEffect effectName
				if ((superclassof effectParam)==textureMap) then (
					hdriTex = convertB2HDRI_SubTex effectParam
					if (hdriTex!=undefined) do setProperty currEffect effectName hdriTex
				) else if ((superclassof effectParam)==material) then (
					convertB2HDRI_SubMtl effectParam
				) else convertB2HDRI_ArrayParameter effectParam
			)
		)
	)
)

--The main function that visits scene objects and replace Bitmaptexture with corresponding VRayHDRI 
--exrConvert:<true|false> - true->converts bitmaps files to EXR/TX files.
--applyOnSceneObjects:<true|false> -> true->allows Bitmaptextures directly assigned to scene nodes to be converted to VRayHDRI
fn VRayBitmap2VRayHDRI exrConvert: applyOnSceneObjects: =(
	convertB2HDRI_ConvertEXR=false
	if (exrConvert!=unsupplied) do (
		if ((classof exrConvert)!=BooleanClass) then (print "Expected Boolean for exrConvert:")
		else convertB2HDRI_ConvertEXR=exrConvert
	)

	local applyOnObjects=false
	if (applyOnSceneObjects!=unsupplied) do (
		applyOnObjects = applyOnSceneObjects
	)

	local listBmpTexs = getClassInstances bitmaptex processChildren:true
	if applyOnObjects then (
		--applyOnSceneObjects set to true allows all Bitmaptextures instances in the scene to be replaced
		for iTex in listBmpTexs do (
			new_tex=convertB2HDRI_Texmap iTex
			if new_tex!=undefined then (
				replaceinstances iTex new_tex
			)
			else format "Converting Bitmap % to VRayHDRI was unsuccessful\n" iTex
		)
	) else (
		--here we must avoid Bitmaptextures directly assigned to scene nodes
		new_hdritexmaps=#()
		orig_bitmaptextures=#()
		listOfConvertedNodesHandles=#()

		for iTex in listBmpTexs do (
			local listOfNodes =  (refs.dependentNodes iTex)
			for iNode in listOfNodes do (
				local i=findItem listOfConvertedNodesHandles iNode.handle
				if i == 0 do (
					convertB2HDRI_Node iNode applyOnObjects
					append listOfConvertedNodesHandles iNode.handle
				)
			)
		)

		for i=1 to meditMaterials.count do (
			local m=meditMaterials[i]
			if (m.superClassID==3072) then ( -- This is a material
				convertB2HDRI_SubMtl m
			) else ( -- This is a texture
				m=convertB2HDRI_SubTex m
				if (m!=undefined) then meditMaterials[i]=m
			)
			updateMTLInMedit meditMaterials[i]
		)

		convertB2HDRI_RenderSettings()
		convertB2HDRI_Environment()
	)
)

rollout convertB2HDRI_Rollout "V-Ray Bitmap to VRayHDRI texmap converter" width: 460 height: 110 (
	label label_message "All scene materials' Bitmap texmaps will be converted to VRayHDRI texmap." width:400 height:13 align:#center offset:[0, 10]

	checkbox check_convertEXR "Convert bitmaps to tiled EXR/TX." checked:false width:400 align:#center offset:[0, 10]
	checkbox check_convertObjectsInScene "Convert All scene Objects' Bitmap texmaps to VRayHDRI texmap." checked:false width:400 align:#center

	progressbar prog_convert width:290 align:#left across:3 offset:[17, 0]
	button btn_start "Convert" width:60 align:#right offset:[83, 0]
	button btn_cancel "Cancel" width:60 align:#right

	label label_progress "" width:150 align:#left offset:[17, -12]

	local cancelled=false
	local convertPath=undefined
	local convertHalf=false
	local convertCompression="zip"
	local convertTilesize=64
	local convertLinear="auto"

	local replaceImgCounter=0
	local replaceImgNums=0

	on btn_start pressed do (
		convertB2HDRI_ConverterErrors=0
		cancelled=false

		if (check_convertEXR.checked) do (
			local ver=getMaxReleaseAsString()
			convertPath = getVRayInstallPath ver
			if useMakeTx then
				CreateDialog convertB2HDRI_MakeTXConverterRollout modal:true
			else
				CreateDialog convertB2HDRI_TEXRConverterRollout modal:true
		)
		if (NOT cancelled) do (
			if (check_convertEXR.checked) do (
				label_progress.text = "Converting in progress..."
				if useMakeTx then
					convertImgToTX()
				else
					convertImgToTiledEXR convertHalf convertCompression convertTilesize convertLinear
			)
			start = timeStamp()
			label_progress.text = "Replacing in progress..."
			replaceImgCounter=0
			replaceImgNums = (getclassinstances Bitmaptexture processChildren:true).count
			vrayBitmap2VRayHDRI exrConvert:check_convertEXR.checked applyOnSceneObjects:check_convertObjectsInScene.checked
			end = timeStamp()
			format "Replacing took % seconds\n" ((end - start) / 1000.0)
			if (convertB2HDRI_ConverterErrors>0) do (
				messageBox ("There were "+(convertB2HDRI_ConverterErrors as String)+" images that did not convert successfully to tiled EXR format.\n They are listed in MAXScript Listener.\n The original images were left in place.") title:"Tiled EXR conversion error"
			)
		)

		destroyDialog convertB2HDRI_Rollout
	)

	on btn_cancel pressed do (
		destroyDialog convertB2HDRI_Rollout
	)
)

--vrayBitmapToVRayHDRI is a script equivalent to 'V-Ray Bitmap to VRayHDRI texmap converter' dialog
--In order to use maxscript function you should specify two arguments if you don't want the function to be exectued with its parameters' default values:
--  txConvert: <true|false> - Converts bitmaps to tiled EXR/TX. The default value is false.
--                                      If you've desided to have texmap conversion to tiled EXR/TX - that means txConvert is set to be true - and conversion tool is 
--                                      'img2tiledexr', 4 specific parameters that corresponds to 'Img2tiledexr tool conversion options' dialog should be initialised.
--                                      convertHalf:<true|false>.  The default value is false.
--                                      convertCompression:<"none"|"rle"|"zips"|"zip"|"piz"|"pxr24">.  The default value is "zip".
--                                      convertTilesize:<32|64|128>.  The default value is 64.
--                                      convertLinear:<"off"|"on"|"auto">.  The default value is "auto".
--                                      If you've missed to do initialization, the default values will be used.
--  applyOnSceneObjects: <true|false> - Converts All scene Objects' Bitmap texmaps to VRayHDRI texmap. false is a default value.

fn vrayBitmapToVRayHDRI txConvert:false applyOnSceneObjects:false
                        convertHalf:true convertCompression:"zip" convertTilesize:64 convertLinear:"auto"= (
	local convertPath=undefined
	convertB2HDRI_ConverterErrors=0
	if (txConvert) do (
		local ver=getMaxReleaseAsString()
		convertPath = getVRayInstallPath ver
		if useMakeTx then
			convertImgToTX()
		else
			convertImgToTiledEXR convertHalf convertCompression convertTilesize convertLinear 
	)

	start = timeStamp()
	--convertB2HDRI_Rollout is used just for consistency, to keep correct value for convertLinear and prevent division by zero in convertB2HDRI_Texmap
	convertB2HDRI_Rollout.replaceImgCounter=0
	convertB2HDRI_Rollout.replaceImgNums = (getclassinstances Bitmaptexture processChildren:true).count
	convertB2HDRI_Rollout.convertLinear = convertLinear
	vrayBitmap2VRayHDRI exrConvert:txConvert applyOnSceneObjects:applyOnSceneObjects
	end = timeStamp()
	format "Replacing took % seconds\n" ((end - start) / 1000.0)
	if (convertB2HDRI_ConverterErrors>0) do (
		messageBox ("There were "+(convertB2HDRI_ConverterErrors as String)+" images that did not convert successfully to tiled EXR format.\n They are listed in MAXScript Listener.\n The original images were left in place.") title:"Tiled EXR conversion error"
	)
)

macroScript VRayProperties
	category:"VRay"
	buttontext:"V-Ray properties"
	tooltip:"Displays the VRay object or light properties"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (doVRayObjectProperties())
)

macroScript VRaySceneConverter
	category:"VRay"
	buttontext:"V-Ray scene converter"
	tooltip:"Convert all scene materials to VRayMtl and all raytraced shadows to VRay shadows"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (CreateDialog vraySceneConverterRollout modal:true)
)

macroScript VRayBitmap2VRayHDRITexmapConverter
	category:"VRay"
	buttonText:"V-Ray Bitmap to VRayHDRI converter"
	tooltip:"Convert all scene materials' Bitmaps to VRayHDRI"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (CreateDialog convertB2HDRI_Rollout modal:true)
)

macroScript VRayMeshExport
	category:"VRay"
	buttontext:"V-Ray mesh export"
	tooltip:"Exports the selected meshes to an external file"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (doVRayMeshExport())
)

macroScript VRayShowVFB
	category:"VRay"
	buttontext:"V-Ray VFB"
	tooltip:"Displays the last rendered V-Ray VFB, if is exists"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (
		r=renderers.production
		if(r!=undefined) do (
			cid=renderers.production.classid
			if (cid[1]==1941615238 and cid[2]==2012806412) do r.showLastVFB()
			if (cid[1]==1770671000 and cid[2]==1323107829) do r.V_Ray_settings.showLastVFB()
		)
	)
)

-- Menu bar V-Ray action items
macroScript VRayProxy
	category:"VRay"
	buttonText:"V-Ray Proxy"
	toolTip: "Create V-Ray Proxy"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayProxy)
)

macroScript VRayFur
	category:"VRay"
	buttonText:"V-Ray Fur"
	toolTip: "Create V-Ray Fur"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer() AND (superClassOf $ == GeometryClass))
	on isVisible do (checkVRayRenderer())

	on execute do (
		if($ == undefined) then warningToSelectGeometry "V-Ray Fur"

		local vrFur = undefined
		theHold.Begin()
		if(superClassOf $ == GeometryClass) then (
			vrFur = VRayFur sourceNode:$ pos:$.pos
			append $.children vrFur
			select vrFur
			setCommandPanelTaskMode #modify
		) else if(isKindOf $ ObjectSet) then (
			for obj in $ do (
				if(superClassOf obj == GeometryClass) do (
					vrFur = VRayFur sourceNode:obj pos:obj.pos
					append obj.children vrFur
				)
			)
			select vrFur
			setCommandPanelTaskMode #modify
		)
		else (
			print "Please select a geometry object to apply V-Ray Fur!"
		)
		theHold.Accept("V-Ray Fur")
	)
)

macroScript VRayPlane
	category:"VRay"
	buttonText:"V-Ray Plane"
	toolTip: "Create V-Ray Plane"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayPlane)
)

macroScript VRaySphere
	category:"VRay"
	buttonText:"V-Ray Sphere"
	toolTip: "Create V-Ray Sphere"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRaySphere)
)

macroScript VRayMetaball
	category:"VRay"
	buttonText:"V-Ray Metaball"
	toolTip: "Create V-Ray Metaball"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayMetaball)
)

macroScript VRayClipper
	category:"VRay"
	buttonText:"V-Ray Clipper"
	toolTip: "Create V-Ray Clipper"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayClipper)
)

macroScript VRayInstancer
	category:"VRay"
	buttonText:"V-Ray Instancer"
	toolTip: "Create V-Ray Instancer"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayInstancer)
)

macroScript VRayVolumeGrid
	category:"VRay"
	buttonText:"V-Ray VolumeGrid"
	toolTip: "Create V-Ray VolumeGrid"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayVolumeGrid)
)

macroScript VRayLight_Plane
	category:"VRay"
	buttonText:"V-Ray Plane Light"
	toolTip: "Create V-Ray Plane Light"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 0))

	on execute do (StartObjectCreation VRayLight newNodeCallback: setType)
)

macroScript VRayLight_Dome
	category:"VRay"
	buttonText:"V-Ray Dome Light"
	toolTip: "Create V-Ray Dome Light"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 1))

	on execute do (StartObjectCreation VRayLight newNodeCallback: setType)
)

macroScript VRayLight_Sphere
	category:"VRay"
	buttonText:"V-Ray Sphere Light"
	toolTip: "Create V-Ray Sphere Light"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 2))

	on execute do (StartObjectCreation VRayLight newNodeCallback: setType)
)

macroScript VRayLight_Mesh
	category:"VRay"
	buttonText:"V-Ray Mesh Light"
	toolTip: "Create V-Ray Mesh Light"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	fn setType obj = (if (classof(obj)==VRayLight) do (obj.type = 3))

	on execute do (StartObjectCreation VRayLight newNodeCallback: setType)
)

macroScript VRayIES
	category:"VRay"
	buttonText:"V-Ray IES Light"
	toolTip: "Create V-Ray IES Light"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if(renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayIES)
)

macroScript VRayAmbientLight
	category:"VRay"
	buttonText:"V-Ray Ambient Light"
	toolTip: "Create V-Ray Ambient Light"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayAmbientLight)
)

macroScript VRaySun
	category:"VRay"
	buttonText:"V-Ray Sun"
	toolTip: "Create V-Ray Sun"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRaySun)
)

macroScript VRayDomeCamera
	category:"VRay"
	buttonText:"V-Ray Dome Camera"
	toolTip: "Create V-Ray Dome Camera"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayDomeCamera)
)

fn vrayCreateVRayPhysicalCameraFromView = (
	--camera variable
	local c = undefined
	local wcol = (color 5 54 179)
	local targdist = viewport.GetFocalDistance()

	-- get viewport tm
	local viewfov = getVIewFOV()
	local cXform = if InverseHighPrecision == undefined then Inverse(viewport.getTM())
					else InverseHighPrecision(viewport.getTM())

	local viewportType = viewport.Gettype()
	theHold.Begin()
	c = VRayPhysicalCamera targeted:false isSelected:on wirecolor:wcol
	c.target_distance = targdist
	c.specify_fov = true
	c.fov = viewfov
	c.transform = cXform
	local viewportCamera = viewport.getCamera()
	if viewportCamera != undefined then (
		if (hasproperty viewportCamera "type") do (
			c.targeted = (viewportCamera.type == #target)
			--that workaround prevents crash when target camera has been created
			--but target node is missing. For example camera created via Targetcamera() 
			--and aligned to viewport there isn't target node.
			--After that if we try to create camera from toolbar, it is expected viewportCamera
			--to have valid target in order to get its position. If viewportCamera.target is undefined
			--then the result is crash. The next implementation ensures viewportCamera to have 
			--target node.
			if c.targeted and viewportCamera.target == undefined then (
				viewportCamera.type=#free
				viewportCamera.type=#target
			)
		)
		if (hasproperty viewportCamera "targeted") do (
			c.targeted = viewportCamera.targeted
			--that workaround prevents crash when target camera has been created
			--but target node is missing. For example camera created via Physical(), vrayPhysicalCamera() 
			--or vraycreatevrayphysicalcamera() and aligned to viewport there isn't target node.
			--After that if we try to create camera from toolbar, it is expected viewportCamera
			--to have valid target in order to get its position. If viewportCamera.target is undefined
			--then the result is crash. The next implementation ensures viewportCamera to have 
			--target node.
			if c.targeted and viewportCamera.target == undefined then (
				viewportCamera.targeted=false
				viewportCamera.targeted=true
			)
		)
	) else (
		if (viewportType == #view_persp_user) do c.targeted = true
	)
	if c.targeted do (
		c.target.wirecolor = wcol
		if viewportCamera != undefined do (
			c.target.pos = viewportCamera.target.pos
			vpTController = viewportCamera.transform.controller
			vrayTController = c.transform.controller
			if vrayTController != undefined and vpTController != undefined do (
				--Transfer of LookAt controller data
				vrayTController.UseTargetAsUpNode = vpTController.UseTargetAsUpNode
				vrayTController.Axis = vpTController.Axis
				vrayTController.Flip = vpTController.Flip
				vrayTController.Position = vpTController.Position
				vrayTController.Roll_Angle = vpTController.Roll_Angle
				vrayTController.Scale = vpTController.Scale
			)
		)
	)
	viewport.setcamera c
	theHold.Accept "Create V-Ray Physical Camera from View"
	-- clears the local variables
	viewfov = undefined
	cXform  = undefined

	c
)

macroScript VRayPhysicalCamera
	category:"VRay"
	buttonText:"V-Ray Physical Camera"
	toolTip: "Create V-Ray Physical Camera"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)
	on IsEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (
		resultCamera = vrayCreateVRayPhysicalCameraFromView()
		if resultCamera == undefined then StartObjectCreation VRayPhysicalCamera
	)
)

macroScript VRayLightMeter
	category:"VRay"
	buttonText:"V-Ray Light Meter"
	toolTip: "Create V-Ray Light Meter"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayLightMeter)
)

macroScript VRayBPTracer
	category:"VRay"
	buttonText:"V-Ray BPTracer"
	toolTip: "Create V-Ray BPTracer"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayBPTracer)
)

macroScript VRayStereoscopic
	category:"VRay"
	buttonText:"V-Ray Stereoscopic"
	toolTip: "Create V-Ray Stereoscopic"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayStereoscopic)
)

macroScript VRayStereoRig
	category:"VRay"
	buttonText:"V-Ray StereoRig"
	toolTip: "Create V-Ray StereoRig"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer())
	on isVisible do (checkVRayRenderer())

	on execute do (StartObjectCreation VRayStereoRig)
)

macroScript VRayDisplacementMod
	category:"VRay"
	buttonText:"V-Ray Displacement Mod"
	toolTip: "V-Ray Displacement Mod"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer() AND (ValidModifier $ VRayDisplacementMod))
	on isVisible do (checkVRayRenderer())

	on execute do (
		if ($==undefined) do (warningToSelectGeometry "Displacement Modifier")

		theHold.begin()
		if ((superClassOf $)==GeometryClass) then (
			if (ValidModifier $ VRayDisplacementMod) do (
				addModifier $ (VRayDisplacementMod())
			)
		) else if (isKindOf $ ObjectSet) then (
			for obj in $ do (
				if (((superClassOf obj)==GeometryClass) AND (ValidModifier obj VRayDisplacementMod)) do (
					addModifier obj (VRayDisplacementMod())
				)
			)
		)
		else (
			print "Please select a geometry object to apply Displacement Modifier!"
		)
		theHold.Accept("V-Ray DisplacementMod")
	)
)

macroScript VRayOrnatrixMod
	category:"VRay"
	buttonText:"V-Ray Ornatrix Mod"
	toolTip: "V-Ray Ornatrix Mod"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer() AND (ValidModifier $ VRayOrnatrixMod))
	on isVisible do (checkVRayRenderer())

	on execute do
	(
		theHold.Begin()
		if ((superClassOf $)==GeometryClass) then (
			if (ValidModifier $ VRayOrnatrixMod) do (
				addModifier $ (VRayOrnatrixMod())
			)
		) else if (isKindOf $ ObjectSet) then (
			for obj in $ do (
				if (((superClassOf obj)==GeometryClass) AND (ValidModifier obj VRayOrnatrixMod)) do (
					addModifier obj (VRayOrnatrixMod())
				)
			)
		)
		else (
			warningToSelectGeometry "Ornatrix Modifier"
		)
		theHold.Accept("V-Ray Ornatrix Mod")
	)
)

macroScript VRayHairFarmMod
	category:"VRay"
	buttonText:"V-Ray HairFarm Mod"
	toolTip: "V-Ray HairFarm Mod"
(
	-- This check implementation is present in all maxscripts, to make them independent on other scirpts
	fn checkVRayRenderer = (
		local enabled=false
		if (renderers.production!=undefined) do (
			cid=renderers.production.classid
			enabled=((cid[1]==1941615238) and (cid[2]==2012806412) or (cid[1]==1770671000) and (cid[2]==1323107829))
		)
		enabled
	)

	on isEnabled do (checkVRayRenderer() AND (ValidModifier $ VRayHairFarmMod))
	on isVisible do (checkVRayRenderer())

	on execute do (
		if ($==undefined) do (warningToSelectGeometry "V-Ray HairFarm")
		theHold.Begin()
		if ((superClassOf $)==GeometryClass) then (
			if (ValidModifier $ VRayHairFarmMod) do (
				addModifier $ (VRayHairFarmMod())
			)
		) else if (isKindOf $ ObjectSet) then (
			for obj in $ do (
				if (((superClassOf obj)==GeometryClass) AND (ValidModifier obj VRayHairFarmMod)) do (
					addModifier obj (VRayHairFarmMod())
				)
			)
		)
		else (
			print "Please select a geometry object to apply V-Ray HairFarm!"
		)
		theHold.Accept("V-Ray HairFarm")
	)
)

-- Macroscript for the Viewport IPR button - visible when the current renderer is V-Ray and we
-- are currently not running Viewport IPR
macroscript VRayViewportIPR
	category:"VRay"
	buttontext:"V-Ray IPR"
	tooltip:"V-Ray Viewport IPR"
(
	on isVisible do (
		if (vrayViewportIPRControl!=undefined) then (
			local viewportVisible=vrayViewportIPRControl 0 0
			local IPRrunning=0
			if (vrayIsRenderingIPR!=undefined) then (
				IPRrunning=vrayIsRenderingIPR()
			)
			viewportVisible and IPRrunning!=2
		) else (
			false
		)
	)

	on isEnabled do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 0 1
		else
			false
	)

	on isChecked do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 0 2
		else
			false
	)

	on Execute do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 0 3
		else
			false
	)
)

-- Macroscript for the Stop Viewport IPR button - visible when there is a viewport IPR running
macroscript VRayViewportIPRStop
	category:"VRay"
	buttontext:"Stop V-Ray IPR"
	tooltip:"V-Ray Viewport IPR"
(
	on isVisible do (
		if (vrayViewportIPRControl!=undefined) then (
			vrayViewportIPRControl 0 0
		) else (
			false
		)
	)

	on isEnabled do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 0 1
		else
			false
	)

	on isChecked do (false)

	on Execute do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 0 3
		else
			false
	)
)

-- Macroscript for the Viewport GPU IPR button - visible when the current renderer is V-Ray GPU and we
-- are currently not running Viewport IPR
macroscript VRayGPUViewportIPR
	category:"VRay"
	buttontext:"V-Ray GPU IPR"
	tooltip:"V-Ray GPU Viewport IPR"
(
	on isVisible do (
		if (vrayViewportIPRControl!=undefined) then (
			local viewportVisible=vrayViewportIPRControl 1 0
			local IPRrunning=0
			if (vrayIsRenderingIPR!=undefined) then (
				IPRrunning=vrayIsRenderingIPR()
			)
			viewportVisible and IPRrunning!=2
		)
		else (
			false
		)
	)

	on isEnabled do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 1 1
		else
			false
	)

	on isChecked do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 1 2
		else
			false
	)

	on Execute do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 1 3
		else
			false
	)
)

-- Macroscript for the Stop Viewport GPU IPR button - visible when there is a viewport IPR running
macroscript VRayGPUViewportIPRStop
	category:"VRay"
	buttontext:"Stop V-Ray GPU IPR"
	tooltip:"V-Ray GPU Viewport IPR"
(
	on isVisible do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 1 0
		else
			false
	)

	on isEnabled do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 1 1
		else
			false
	)

	on isChecked do (false)

	on Execute do (
		if (vrayViewportIPRControl!=undefined) then
			vrayViewportIPRControl 1 3
		else
			false
	)
)

-- Debug shading

-- Return true if the menu item for the specified mode should be checked
fn getVRayDebugShadingChecked mode=(
	local res=vfbcontrol #debugshading #debugshadingmode
	(res!=undefined) and (res.count==2) and (res[1]!=0) and (res[2]==mode)
)

-- Return true if the debug shading menus should be enabled - only while IPR is running
fn getVRayDebugShadingEnabled=(
	if (vrayIsRenderingIPR!=undefined) then
		vrayIsRenderingIPR()
	else
		false
)

-- Set the specified mode. If the mode is already selected - turn off debug shading
fn setVRayDebugShadingMode mode=(
	local res=vfbcontrol #debugshading #debugshadingmode #debugshadinglockedselection
	if (res!=undefined and res.count==3) then (
		if (res[2]==mode and res[3]!=1) then (
			-- clicking on the current mode should just invert enabled.
			vfbcontrol #debugshading (not (res[1]>0))
		) else (
			-- different mode - set the mode and enable debug shading
			vfbcontrol #debugshading true
			vfbcontrol #debugshadingmode (mode)
		)
		true
	) else (
		false
	)
)

-- Macro scripts for the various modes.
macroscript VRayIsolateSelect
	category:"VRayDebugShading"
	buttontext:"Isolate Selected"
	tooltip:"V-Ray Viewport Debug Shading Isolate Selected"
(
	-- the debug shading mode of this macroscript
	local ourMode=0

	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)

	on isChecked do (
		-- special case - we have to check for locked selection too
		local lockedSel=vfbcontrol #debugshadinglockedselection
		local isLocked=(lockedSel!=undefined and lockedSel[1]!=0)
		if (getVRayDebugShadingChecked != undefined) then (
			(getVRayDebugShadingChecked (ourMode)) and not isLocked
		)
		else (
			false
		)
	)
	on Execute do (setVRayDebugShadingMode (ourMode))
)

macroscript VRayIsolateSelectLocked
	category:"VRayDebugShading"
	buttontext:"Isolate Locked Selection"
	tooltip:"V-Ray Viewport Debug Shading Isolate Locked Selection"
(
	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)

	on isChecked do (
		res=vfbcontrol #debugshadinglockedselection
		if (res!=undefined) then (
			res[1] != 0
		) else (
			false
		)
	)

	on Execute do (
		res=vfbcontrol #debugshadinglockedselection
		if (res != undefined) do
			vfbcontrol #debugshadinglockedselection (not (res[1]>0))
	)
)

macroscript VRayDebugShadingLighting
	category:"VRayDebugShading"
	buttontext:"Lighting"
	tooltip:"V-Ray Viewport Debug Shading Lighting"
(
	-- the debug shading mode of this macroscript
	local ourMode=1

	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)
	on isChecked do (
		if (getVRayDebugShadingChecked != undefined) then (
			getVRayDebugShadingChecked (ourMode)
		)
		else (
			false
		)
	)
	on Execute do (setVRayDebugShadingMode (ourMode))
)

macroscript VRayDebugShadingAO
	category:"VRayDebugShading"
	buttontext:"Ambient Occlusion"
	tooltip:"V-Ray Viewport Debug Shading Ambient Occlusion"
(
	-- the debug shading mode of this macroscript
	local ourMode=2

	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)
	on isChecked do (
		if (getVRayDebugShadingChecked != undefined) then (
			getVRayDebugShadingChecked (ourMode)
		)
		else (
			false
		)
	)
	on Execute do (setVRayDebugShadingMode (ourMode))
)

macroscript VRayDebugShadingWireframe
	category:"VRayDebugShading"
	buttontext:"Wireframe"
	tooltip:"V-Ray Viewport Debug Shading Wireframe"
(
	-- the debug shading mode of this macroscript
	local ourMode=3

	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)
	on isChecked do (
		if (getVRayDebugShadingChecked != undefined) then (
			getVRayDebugShadingChecked (ourMode)
		)
		else (
			false
		)
	)
	on Execute do (setVRayDebugShadingMode (ourMode))
)

macroscript VRayDebugShadingNormals
	category:"VRayDebugShading"
	buttontext:"Normals"
	tooltip:"V-Ray Viewport Debug Shading Normals"
(
	-- the debug shading mode of this macroscript
	local ourMode=4

	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)
	on isChecked do (
		if (getVRayDebugShadingChecked != undefined) then (
			getVRayDebugShadingChecked (ourMode)
		)
		else (
			false
		)
	)
	on Execute do (setVRayDebugShadingMode (ourMode))
)

macroscript VRayDebugShadingUVs
	category:"VRayDebugShading"
	buttontext:"UVs"
	tooltip:"V-Ray Viewport Debug Shading UVs"
(
	-- the debug shading mode of this macroscript
	local ourMode=5

	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)
	on isChecked do (
		if (getVRayDebugShadingChecked != undefined) then (
			getVRayDebugShadingChecked (ourMode)
		)
		else (
			false
		)
	)
	on Execute do (setVRayDebugShadingMode (ourMode))
)

macroscript VRayDebugShadingBary
	category:"VRayDebugShading"
	buttontext:"Barycentric coordinates"
	tooltip:"V-Ray Viewport Debug Shading Barycentric coordinates"
(
	-- the debug shading mode of this macroscript
	local ourMode=6

	on isVisible do (true)
	on isEnabled do (
		if (getVRayDebugShadingEnabled != undefined) then (
			getVRayDebugShadingEnabled() != 0
		) else (
			false
		)
	)
	on isChecked do (
		if (getVRayDebugShadingChecked != undefined) then (
			getVRayDebugShadingChecked (ourMode)
		)
		else (
			false
		)
	)
	on Execute do (setVRayDebugShadingMode (ourMode))
)

-- Return the name of the Debug shading menu
fn getVRayDebugShadingMenuName=("V-Ray IPR")
fn getVRayGPUDebugShadingMenuName=("V-Ray GPU IPR")

-- Register the debug shading Viewport IPR menu and submenus
-- param updateMenu - should we call menuMan.updateMenuBar() or not
-- return true if there were menus added the caller should call menuMan.updateMenuBar()
fn registerDbgShading updateMenu = (
	if (checkProductionRenderer()) then (
		local isVRayIPR=true -- by default assume V-Ray IPR
		if (vrayViewportIPRControl!=undefined) then
			isVRayIPR=vrayViewportIPRControl 0 0

		local nitrousMenu=getShadingViewportNitrousLabelMenu()

		-- Decide how we should name the menu - V-Ray IPR or V-Ray GPU IPR
		local menuName
		if (isVRayIPR) then (
			menuName=getVRayDebugShadingMenuName()
		) else (
			menuName=getVRayGPUDebugShadingMenuName()
		)

		local menuDebug = menuMan.createMenu menuName
		-- Add items to the submenu
		if (menuDebug!=undefined) then (
			if (isVRayIPR) then (
				appendMenuItem menuDebug "VRayViewportIPRStop" "VRay"
			)
			else (
				appendMenuItem menuDebug "VRayGPUViewportIPRStop" "VRay"
			)
			menuDebug.addItem (menuMan.createSeparatorItem()) -1

			appendMenuItem menuDebug "VRayIsolateSelect" "VRayDebugShading"
			appendMenuItem menuDebug "VRayIsolateSelectLocked" "VRayDebugShading"
			appendMenuItem menuDebug "VRayDebugShadingLighting" "VRayDebugShading"
			appendMenuItem menuDebug "VRayDebugShadingAO" "VRayDebugShading"
			appendMenuItem menuDebug "VRayDebugShadingWireframe" "VRayDebugShading"
			appendMenuItem menuDebug "VRayDebugShadingNormals" "VRayDebugShading"
			appendMenuItem menuDebug "VRayDebugShadingUVs" "VRayDebugShading"
			appendMenuItem menuDebug "VRayDebugShadingBary" "VRayDebugShading"

			local subMenu1 = menuMan.createSubMenuItem "V-Ray Debug Shading" menuDebug

			local separator=findFirstSeparatorIdx nitrousMenu
			nitrousMenu.addItem subMenu1 separator

			if (updateMenu) then (
				menuMan.updateMenuBar()
				false
			)
		) else (
			false
		)
	)
	else (
		false
	)
)

-- Unregister debug shading viewport menus.
-- param updateMenu - should we call menuMan.updateMenuBar() or not
-- return true if there were menus added the caller should call menuMan.updateMenuBar()
fn unregisterDbgShading updateMenu =(
	local vrayMenu=getShadingViewportNitrousLabelMenu()
	local updateNeeded=false

	-- Try to remove the V-Ray IPR menus
	item=findSubMenuItem vrayMenu (getVRayDebugShadingMenuName())
	if (item!=undefined) do (
		vrayMenu.removeItem item
		updateNeeded = true
	)

	-- also remove from the main menu
	item=menuMan.findMenu (getVRayDebugShadingMenuName())
	if (item!=undefined) do (
		menuMan.unRegisterMenu item
		updateNeeded = true
	)

	-- Now try to remove the V-Ray GPU menus too
	item=findSubMenuItem vrayMenu (getVRayGPUDebugShadingMenuName())
	if (item!=undefined) do (
		vrayMenu.removeItem item
		updateNeeded = true
	)

	-- also remove from the main menu
	item=menuMan.findMenu (getVRayGPUDebugShadingMenuName())
	if (item!=undefined) do (
		menuMan.unRegisterMenu item
		updateNeeded = true
	)

	if (updateMenu and updateNeeded) then (
		menuMan.updateMenuBar()
		updateNeeded=false
	)

	updateNeeded
)

-- End debug shading

-- Localized menu names
menuName_Tools=#("Ferramentas", "Tools", "Extras", "Outils", "ツール(T)", "도구(T)", "工具(T)")
menuName_Tools_LightLister=#("Criador de lista de luz...", "Light Lister...", "Lichtliste...", "Liste de lumières...", "ライト リスト...", "라이트 리스트자...", "灯光列表...")
menuName_Create=#("Criar", "Create", "Erstellen", "Créer", "作成(C)", "만들기(C)", "创建(C)")
menuName_Create_Lights=#("Luzes", "Lights", "Lichtquellen", "Lumières", "ライト(L)", "라이트(L)", "灯光(L)")
menuName_Create_Cameras=#("Câmeras", "Cameras", "Kameras", "Caméras", "カメラ(C)", "카메라(C)", "摄影机(C)")
menuName_Create_Helpers=#("Ajudas", "Helpers", "Helfer", "Assistants", "ヘルパー", "도우미", "辅助对象")
menuName_Create_Systems=#("Sistemas", "Systems", "Systèmes", "Systeme", "システム", "시스템", "系统")
menuName_Modifiers=#("Modificadores", "Modifiers", "Modifikatoren", "Modificateurs", "モディファイヤ", "수정자", "修改器")
mainMenuBar_Rendering=#("Renderização", "Rendering", "Rendern", "Rendu", "レンダリング(R)", "렌더링(R)", "渲染(R)")

menuName_Enhanced_LightingCameras_Tools=#("Ferramentas (iluminação/conjunto de câmeras)", "Tools (Lighting/Cameras Set)","Hilfsmittel (Beleuchtung/Kamera-Satz)","Outils (Eclairage/caméras)","ツール (ライト/カメラ セット)","도구(조명/카메라 세트)","工具(照明/摄影机设置)")
menuName_Enhanced_Rendering_Preview=#("Captura de tela", "Screen Capture","Bildschirmausdruck","Capture d'écran","スクリーン キャプチャ","화면 캡처","屏幕捕获")
menuName_Enhanced_Objects=#("Objetos", "Objects","Objekte","Objets","オブジェクト(&O)","오브젝트(&O)","对象(&O)")
menuName_Enhanced_LightingCameras=#("Iluminação/câmeras", "Lighting/Cameras","Beleuchtung/Kameras","Eclairage/Caméras","ライト/カメラ(&L)","조명/카메라(&L)","照明/摄影机(&L)")
menuName_Enhanced_Materials=#("Materiais", "Materials", "Materialien", "Matériaux", "マテリアル(&T)", "재질(&T)", "材质(&T)")
menuName_Enhanced_Materials_Tools=#("Ferramentas (conjunto de materiais)", "Tools (Material Set)", "Hilfsmittel (Materialsatz)", "Outils (Matériaux)", "ツール (マテリアル セット)(&T)", "도구(재질 세트)(&T)", "工具(材质设置)(&T)")

-- Menu editing utility functions
-- Append menu item to a menu object;
-- Parameters: Menu
--             Item's MacroScript Name,
--             Item's MacroScript Category,
fn appendMenuItem mobj msname mscat = mobj.addItem (menuMan.createActionItem msname mscat) -1

-- Insert menu item to a menu object at specified position;
-- Parameters: Menu
--             Item's MacroScript Name,
--             Item's MacroScript Category,
--             Item's insert position
fn InsertMenuItem mobj msname mscat idx = mobj.addItem (menuMan.createActionItem msname mscat) idx

-- Insert submenu item:
-- Parameters: Menu
--             Menu Name
--             Submenu Object
--             Menu Index
fn insertSubmenuItem mobj mname sobj mindex = (
	local subMenuItem = menuMan.createSubMenuItem mname sobj
	subMenuItem.setUseCustomTitle true
	mobj.addItem subMenuItem mindex
)

-- Return index of the first separator in a menu, if no separator last element's index is returned
-- Parameters:	Menu
fn findFirstSeparatorIdx mobj = (
	local i = 1
	while ((i <= mobj.numItems()) and (not((mobj.getItem i).getIsSeparator()))) do (
		i += 1
	)

	i
)

fn findLastSeparatorIdx mobj = (
	local i = mobj.numItems()
	while ((i <= 1) and (not((mobj.getItem i).getIsSeparator()))) do (
		i -= 1
	)

	i
)

-- Compare two strings - the first may contain & for key accelerators
-- Parameters string1 String that might contain accelerators
--            string2 String to compare to. Must not contain accelerators
fn compareStringsWithAccelerators string1 string2=(
	local string1Elements = filterString (string1) "&"
	local string1All=""
	local j
	for j = 1 to string1Elements.count do (
		string1All=append string1All string1Elements[j]
	)

	string1All == string2
)

-- Return menu item from a menu, found by title
-- Parameters: Menu
--             Item Name
fn findMenuItem mobj iname = (
	local item = undefined
	local i
	for i = 1 to mobj.numItems() do (
		if ((compareStringsWithAccelerators ((mobj.getItem i).getTitle()) iname)) do (
			item = mobj.getItem i
		)
	)

	item
)

-- Return menu item's index from a menu, found by title
-- Parameters: Menu
--             Item Name
fn findMenuItemIdx mobj iname = (
	local item = undefined
	local i
	for i = 1 to mobj.numItems() do (
		if ((compareStringsWithAccelerators ((mobj.getItem i).getTitle()) iname)) do (
			item = i
		)
	)

	item
)

-- Return menu item from a menu, found by title. Finds only menu items that have sub menus
-- Parameters: Menu
--             Item Name
fn findSubMenuItem mobj iname = (
	local item = undefined
	local i
	for i = 1 to mobj.numItems() do (
		local currItem=mobj.getItem i
		if ((compareStringsWithAccelerators (currItem.getTitle()) iname) and currItem.getSubMenu() != undefined) do (
			item = currItem
		)
	)

	item
)

-- Remove all menu item's occurences in a menu, found by title
-- and remaining adjacent separators.
-- Parameters: mobj menu object
--             iname item title
-- Returns: true if the menu is changed and the menu bar should be updated
fn vrayMenuRemoveItemsByTitle mobj iname = (
	local shouldUpdate=false
	if (mobj!=undefined) do (
		local idx=undefined
		do (
			idx=findMenuItemIdx mobj iname
			if (idx!=undefined) do (
				mobj.removeItemByPosition idx

				local prevItem=undefined
				local nextItem=undefined
				if (idx>1) do prevItem=mobj.getItem (idx-1)
				if (idx<=mobj.numItems()) do nextItem=mobj.getItem idx
				if (prevItem!=undefined and nextItem!=undefined) then (
					if (prevItem.getIsSeparator() and nextItem.getIsSeparator()) do mobj.removeItemByPosition (idx-1)
				)
				else (
					if (nextItem!=undefined and nextItem.getIsSeparator()) do mobj.removeItemByPosition idx
					if (prevItem!=undefined and prevItem.getIsSeparator()) do mobj.removeItemByPosition (idx-1)
				)

				shouldUpdate=true
			)
		) while (idx!=undefined)
	)
	shouldUpdate
)

fn getShadingViewportNitrousLabelMenu = (
	local nitrousMenu=undefined
	menuNames=#(
		"Vistas - Menu de etiqueta do Nitrous de viewport de sombreamento",   -- Brasilian Portuguese
		"Views - Shading Viewport Nitrous Label Menu",                        -- English
		"Vues - Menu Etiquette fenêtre ombrage de Nitrous",                   -- French
		"Ansichten - Ansichtsfensterschattierung: Nitrous-Beschriftungsmenü", -- German
		"ビュー - シェーディング ビューポート Nitrous ラベル メニュー",                              -- Japanese
		"보기 - 음영 처리 뷰포트 Nitrous 레이블 메뉴",                        -- Korean
		"视图 - 明暗处理视口 Nitrous 标签菜单"                                -- Simplified Chinese
	)
	for i=1 to menuNames.count do (
		nitrousMenu=menuMan.findMenu menuNames[i]
		if nitrousMenu!=undefined do exit
	)
	nitrousMenu
)

-- Register V-Ray objects in menu bar
fn registerVRayMenuBarItems = (
	local updateEnable = false
	local subMenu -- temporary submenu object
	local menuIndex -- temporary index used for item positioning calculation

	local mainMenuBar = menuMan.getMainMenuBar()

	local toolsMenuItem
	for i=1 to menuName_Tools.count do (if (toolsMenuItem==undefined) do (toolsMenuItem=findMenuItem mainMenuBar menuName_Tools[i]))
	if (toolsMenuItem != undefined) do (
		local toolsMenu = toolsMenuItem.getSubMenu()

		-- Add Light Lister item
		if ((findMenuItem toolsMenu "V-Ray Light Lister") == undefined) do (
			local lightListerIndex
			for i=1 to menuName_Tools_LightLister.count do (if (lightListerIndex==undefined) do (lightListerIndex=findMenuItemIdx toolsMenu menuName_Tools_LightLister[i]))
			if (lightListerIndex != undefined) then (
				lightListerIndex += 1
			)
			else (
				lightListerIndex = -1
			)
			toolsMenu.addItem (menuMan.createActionItem "VRayLightLister" "VRay") lightListerIndex
			updateEnable = true
		)

		-- Add Vismat converter
		if ((findMenuItem toolsMenu "V-Ray vrmat converter") == undefined) do (
			toolsMenu.addItem (menuMan.createSeparatorItem()) -1
			toolsMenu.addItem (menuMan.createActionItem "VRayVismatConverter" "VRay") -1
			updateEnable = true
		)
	)

	local createMenuItem
	for i=1 to menuName_Create.count do (if (createMenuItem==undefined) do (createMenuItem=findMenuItem mainMenuBar menuName_Create[i]))
	if (createMenuItem != undefined) do (
		local createMenu = createMenuItem.getSubMenu()

		-- Add geometry objects
		if ((findMenuItem createMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Create - V-Ray Primitives"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayProxy" "VRay"
			appendMenuItem subMenu "VRayFur" "VRay"
			appendMenuItem subMenu "VRayMetaball" "VRay"
			appendMenuItem subMenu "VRaySphere" "VRay"
			appendMenuItem subMenu "VRayClipper" "VRay"
			appendMenuItem subMenu "VRayInstancer" "VRay"
			appendMenuItem subMenu "VRayPlane" "VRay"
			appendMenuItem subMenu "VRayVolumeGrid" "VRay"

			-- Add submenu before the first separator
			menuIndex = findFirstSeparatorIdx createMenu
			insertSubmenuItem createMenu "V-Ray" subMenu menuIndex
			updateEnable = true
		)

		-- Add light objects
		local lightsSubMenuItem
		for i=1 to menuName_Create_Lights.count do (if (lightsSubMenuItem==undefined) do (lightsSubMenuItem=findMenuItem createMenu menuName_Create_Lights[i]))
		if (lightsSubMenuItem != undefined) do (
			local lightsSubMenu = lightsSubMenuItem.getSubMenu()
			if ((findMenuItem lightsSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Lights.V-Ray Lights"

				-- Add items to the submenu
				appendMenuItem subMenu "VRayLight_Plane" "VRay"
				appendMenuItem subMenu "VRayLight_Dome" "VRay"
				appendMenuItem subMenu "VRayLight_Sphere" "VRay"
				appendMenuItem subMenu "VRayLight_Mesh" "VRay"
				appendMenuItem subMenu "VRayIES" "VRay"
				appendMenuItem subMenu "VRayAmbientLight" "VRay"
				appendMenuItem subMenu "VRaySun" "VRay"

				-- Add submenu before the first separator
				menuIndex = findFirstSeparatorIdx lightsSubMenu
				insertSubmenuItem lightsSubMenu "V-Ray" subMenu menuIndex
				updateEnable = true
			)
		)


		-- Add camera objects
		local camerasSubMenuItem
		for i=1 to menuName_Create_Cameras.count do (if (camerasSubMenuItem==undefined) do (camerasSubMenuItem=findMenuItem createMenu menuName_Create_Cameras[i]))
		if (camerasSubMenuItem != undefined) do (
			local camerasSubMenu = camerasSubMenuItem.getSubMenu()
			if ((findMenuItem camerasSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Cameras.V-Ray Cameras"

				-- Add items to the submenu
				appendMenuItem subMenu "VRayDomeCamera" "VRay"
				if (((maxVersion())[1]) < 18000) do (
					appendMenuItem subMenu "VRayPhysicalCamera" "VRay"
				)

				-- Add submenu before the first separator
				menuIndex = findFirstSeparatorIdx camerasSubMenu
				insertSubmenuItem camerasSubMenu "V-Ray" subMenu menuIndex
				updateEnable = true
			)
		)


		-- Add helper objects
		local helpersSubMenuItem
		for i=1 to menuName_Create_Helpers.count do (if (helpersSubMenuItem==undefined) do (helpersSubMenuItem=findMenuItem createMenu menuName_Create_Helpers[i]))
		if (helpersSubMenuItem != undefined) do (
			local helpersSubMenu = helpersSubMenuItem.getSubMenu()
			if ((findMenuItem helpersSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Helpers.V-Ray Helpers"

				-- Add items to the submenu
				appendMenuItem subMenu "VRayLightMeter" "VRay"
				appendMenuItem subMenu "VRayBPTracer" "VRay"
				appendMenuItem subMenu "VRayStereoscopic" "VRay"

				-- Add submenu at the end
				helpersSubMenu.addItem (menuMan.createSeparatorItem()) -1
				insertSubmenuItem helpersSubMenu "V-Ray" subMenu -1
				updateEnable = true
			)
		)

		-- Add systems objects
		local systemsSubMenuItem
		for i=1 to menuName_Create_Systems.count do (if (systemsSubMenuItem==undefined) do (systemsSubMenuItem=findMenuItem createMenu menuName_Create_Systems[i]))
		if (systemsSubMenuItem != undefined) do (
			local systemsSubMenu = systemsSubMenuItem.getSubMenu()
			if ((findMenuItem systemsSubMenu "V-Ray") == undefined) do (
				subMenu = menuMan.createMenu "Create - Systems.V-Ray Systems"

				-- Add items to the submenu
				appendMenuItem subMenu "VRayStereoRig" "VRay"

				-- Add submenu at the end
				systemsSubMenu.addItem (menuMan.createSeparatorItem()) -1
				insertSubmenuItem systemsSubMenu "V-Ray" subMenu -1
				updateEnable = true
			)
		)
	)

	local modMenuItem
	for i=1 to menuName_Modifiers.count do (if (modMenuItem==undefined) do (modMenuItem=findMenuItem mainMenuBar menuName_Modifiers[i]))
	if (modMenuItem != undefined) do (
		local modMenu = modMenuItem.getSubMenu()

		-- Add modifiers
		if ((findMenuItem modMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Modifiers - V-Ray"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayDisplacementMod" "VRay"
			appendMenuItem subMenu "VRayOrnatrixMod" "VRay"
			appendMenuItem subMenu "VRayHairFarmMod" "VRay"

			modMenu.addItem (menuMan.createSeparatorItem()) -1
			insertSubmenuItem modMenu "V-Ray" subMenu -1
			updateEnable = true
		)
	)

	local nitrousMenu=getShadingViewportNitrousLabelMenu()
	if ((nitrousMenu!=undefined) and (((maxVersion())[1]) >= 18900)) do (
		-- Unregister any previous debug shading menus
		if ((unregisterDbgShading false)) do (
			updateEnable=true
		)

		local item=findMenuItem nitrousMenu "V-Ray IPR"
		if (item==undefined) do (
			local separator=findFirstSeparatorIdx nitrousMenu
			local item=menuMan.createActionItem "VRayViewportIPR" "VRay"
			nitrousMenu.addItem item separator
			updateEnable = true
		)
		item=findMenuItem nitrousMenu "V-Ray GPU IPR"
		if (item==undefined) do (
			local separator=findFirstSeparatorIdx nitrousMenu
			local item=menuMan.createActionItem "VRayGPUViewportIPR" "VRay"
			nitrousMenu.addItem item separator
			updateEnable = true
		)
	)

	-- Update menu to show all newly added items
	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
-- ENHANCED MENUBAR
listLanguageIdxs = #("PTB", "ENU", "DEU", "FRA", "JPN", "KOR", "CHS")

-- Register V-Ray objects in enhanced menu bar
fn registerVRayEnhancedMenuBarItems = (

	local updateEnable = false
	local subMenu -- temporary submenu object
	local menuIndex -- temporary index used for item positioning calculation
	local subMenuItem
	local separatorIndex

	local mainMenuBar = menuMan.getMainMenuBar()

	--Get Language index in order to get corresponding to language, menu/item name
	local listLangInfo = sysinfo.GetMaxLanguage() -- #(9, 1, "ENU", "English (United States)", "en-US") - it is used the third element of language info list
	local languageIndex = findItem listLanguageIdxs listLangInfo[3]

	-- Add Light Lister item
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_LightingCameras[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_LightingCameras_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMenu = subMenuItem.getSubMenu()

		if ((findMenuItem toolsMenu "V-Ray Light Lister") == undefined) do (
			menuIndex = findMenuItemIdx toolsMenu menuName_Tools_LightLister[languageIndex]
			if (menuIndex != undefined) then (
				menuIndex += 1
			)
			else (
				menuIndex = -1
			)

			toolsMenu.addItem (menuMan.createActionItem "VRayLightLister" "VRay")  menuIndex
			updateEnable = true
		)
	)

	-- Add geometry objects vray menu
	local subMenuItemObjects =  findMenuItem mainMenuBar menuName_Enhanced_Objects[languageIndex]
	if (subMenuItemObjects != undefined) do (
		local mainObjMenu = subMenuItemObjects.getSubMenu()
		if ((findMenuItem mainObjMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Create - V-Ray Primitives"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayProxy" "VRay"
			appendMenuItem subMenu "VRayFur" "VRay"
			appendMenuItem subMenu "VRayMetaball" "VRay"
			appendMenuItem subMenu "VRaySphere" "VRay"
			appendMenuItem subMenu "VRayClipper" "VRay"
			appendMenuItem subMenu "VRayInstancer" "VRay"
			appendMenuItem subMenu "VRayPlane" "VRay"
			appendMenuItem subMenu "VRayVolumeGrid" "VRay"

			-- Add submenu before the first separator
			menuIndex = findFirstSeparatorIdx mainObjMenu
			insertSubmenuItem mainObjMenu "V-Ray" subMenu menuIndex
			updateEnable = true
		)
	)

	-- Add light objects to  (Objects - Enhanced and Lighting/Cameras - Enhanced)Lights menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Lights[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local lightsMenu = subMenuItem.getSubMenu()
		local lastSeparatorIdx = findLastSeparatorIdx lightsMenu

		if (lastSeparatorIdx <= 1) then lastSeparatorIdx = -1
		else lastSeparatorIdx -= 1

		lightsMenu.addItem (menuMan.createSeparatorItem()) lastSeparatorIdx
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Plane Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Plane" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Dome Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Dome" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Sphere Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Sphere" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Mesh Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayLight_Mesh" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray IES Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayIES" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Ambient Light") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRayAmbientLight" "VRay") lastSeparatorIdx
			updateEnable = true
		)
		lastSeparatorIdx += 1
		if ((findMenuItem lightsMenu "V-Ray Sun") == undefined) do (
			lightsMenu.addItem (menuMan.createActionItem "VRaySun" "VRay") lastSeparatorIdx
			updateEnable = true
		)
	)

	-- Add camera objects to (Objects - Enhanced and Lighting/Cameras - Enhanced)/Cameras menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Cameras[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local camerasMenu = subMenuItem.getSubMenu()
		separatorIndex = findFirstSeparatorIdx camerasMenu
		camerasMenu.addItem (menuMan.createSeparatorItem()) separatorIndex
		if ((findMenuItem camerasMenu "V-Ray Dome Camera") == undefined) do (
			separatorIndex += 1
			camerasMenu.addItem (menuMan.createActionItem "VRayDomeCamera" "VRay") separatorIndex
			updateEnable = true
		)
		if (((maxVersion())[1]) < 18000) do (
			if ((findMenuItem camerasMenu "V-Ray Physical Camera") == undefined) do (
				separatorIndex += 1
				camerasMenu.addItem (menuMan.createActionItem "VRayPhysicalCamera" "VRay") separatorIndex
				updateEnable = true
			)
		)
	)


	-- Add helper objects vray menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Helpers[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local helpersMenu = subMenuItem.getSubMenu()
		if ((findMenuItem helpersMenu "V-Ray Light Meter") == undefined) do (
			helpersMenu.addItem (menuMan.createSeparatorItem()) -1
			helpersMenu.addItem (menuMan.createActionItem "VRayLightMeter" "VRay") -1
			updateEnable = true
		)
		if ((findMenuItem helpersMenu "V-Ray BPTracer") == undefined) do (
			helpersMenu.addItem (menuMan.createSeparatorItem()) -1
			helpersMenu.addItem (menuMan.createActionItem "VRayBPTracer" "VRay") -1
			updateEnable = true
		)
		if ((findMenuItem helpersMenu "V-Ray Stereoscopic") == undefined) do (
			helpersMenu.addItem (menuMan.createSeparatorItem()) -1
			helpersMenu.addItem (menuMan.createActionItem "VRayStereoscopic" "VRay") -1
			updateEnable = true
		)
	)

	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	-- Add modifiers vray menu
	subMenuItem = findMenuItem mainMenuBar menuName_Modifiers[languageIndex]
	if (subMenuItem != undefined) do (
		local modMenu = subMenuItem.getSubMenu()
		if ((findMenuItem modMenu "V-Ray") == undefined) do (
			subMenu = menuMan.createMenu "Modifiers - V-Ray"
			-- Add items to the submenu
			appendMenuItem subMenu "VRayDisplacementMod" "VRay"
			appendMenuItem subMenu "VRayOrnatrixMod" "VRay"

			modMenu.addItem (menuMan.createSeparatorItem()) -1
			insertSubmenuItem modMenu "V-Ray" subMenu -1
			updateEnable = true
		)
	)

	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	-- Add vray vismat convertor
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_Materials[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_Materials_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMaterialMenu = subMenuItem.getSubMenu()
		if ((findMenuItem toolsMaterialMenu "V-Ray vrmat converter") == undefined) do (

			-- Add item
			toolsMaterialMenu.addItem (menuMan.createSeparatorItem()) -1
			toolsMaterialMenu.addItem (menuMan.createActionItem "VRayVismatConverter" "VRay") -1

			updateEnable = true

		)
	)

	-- Update menu to show all newly added items
	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

fn registerVRayQuadMenuItems = (
	local quadMenu=menuMan.getViewportRightClickMenu #nonePressed
	if (quadMenu!=undefined) do (
		local menu=quadMenu.getMenu 1
		if (menu!=undefined) do (
			-- add VRayProperties
			local menuIndex=findMenuItemIdx menu "V-Ray properties"
			if (menuIndex == undefined) do (
				appendMenuItem menu "VRayProperties" "VRay"
			)
			-- add VRayShowVFB
			menuIndex=findMenuItemIdx menu "V-Ray VFB"
			if (menuIndex == undefined) do (
				local menuVRPropertiesIdx=findMenuItemIdx menu "V-Ray properties"
				if (menuVRPropertiesIdx != undefined) then (
					local vrayVFB=menuMan.createActionItem "VRayShowVFB" "VRay"
					menu.addItem vrayVFB (menuVRPropertiesIdx + 1)
				) else (
					appendMenuItem menu "VRayShowVFB" "VRay"
				)
			)
			-- add VRaySceneConverter
			menuIndex=findMenuItemIdx menu "V-Ray scene converter"
			if (menuIndex == undefined) do (
				local menuVRVFBIdx=findMenuItemIdx menu "V-Ray VFB"
				if (menuVRVFBIdx != undefined) then (
					local vraySceneConvert=menuMan.createActionItem "VRaySceneConverter" "VRay"
					menu.addItem vraySceneConvert (menuVRVFBIdx + 1)
				) else (
					appendMenuItem menu "VRaySceneConverter" "VRay"
				)
			)
			-- add Bitmap2VRayHDRI texture converter
			menuIndex=findMenuItemIdx menu "V-Ray Bitmap to VRayHDRI converter"
			if (menuIndex == undefined) do (
				local menuVRSceneConverterIdx=findMenuItemIdx menu "V-Ray scene converter"
				if (menuVRSceneConverterIdx != undefined) then (
					local vrBitmap2VRayHDRI=menuMan.createActionItem "VRayBitmap2VRayHDRITexmapConverter" "VRay"
					menu.addItem vrBitmap2VRayHDRI (menuVRSceneConverterIdx + 1)
				) else (
					appendMenuItem menu "VRayBitmap2VRayHDRITexmapConverter" "VRay"
				)
			)
			-- add VRayMeshExport
			menuIndex=findMenuItemIdx menu "V-Ray mesh export"
			if (menuIndex == undefined) do (
				local menuVRBmp2HDRIConverterIdx=findMenuItemIdx menu "V-Ray Bitmap to VRayHDRI converter"
				if (menuVRBmp2HDRIConverterIdx != undefined) then (
					local vrayMeshExp=menuMan.createActionItem "VRayMeshExport" "VRay"
					menu.addItem vrayMeshExp (menuVRBmp2HDRIConverterIdx + 1)
				) else (
					appendMenuItem menu "VRayMeshExport" "VRay"
				)
			)
			-- add vrsceneExport
			menuIndex=findMenuItemIdx menu ".vrscene exporter"
			if (menuIndex == undefined) do (
				local menuVRMeshExpIdx=findMenuItemIdx menu "V-Ray mesh export"
				if (menuVRMeshExpIdx != undefined) then (
					local vrayVRSceneExp=menuMan.createActionItem "vrsceneExport" "VRay"
					menu.addItem vrayVRSceneExp (menuVRMeshExpIdx + 1)
				) else (
					appendMenuItem menu "vrsceneExport" "VRay"
				)
			)
		)
	)
)

-- Unregister V-Ray objects from menu bar
fn unregisterVRayMenuBarItems = (
	local mainMenuBar = menuMan.getMainMenuBar()
	local vrayMenuItemIdx

	local updateEnable = false

	local toolsMenuItem
	for i=1 to menuName_Tools.count do (if (toolsMenuItem==undefined) do (toolsMenuItem=findMenuItem mainMenuBar menuName_Tools[i]))
	if (toolsMenuItem != undefined) do (
		local toolsMenu = toolsMenuItem.getSubMenu()
		if (toolsMenu != undefined) do (
			if (vrayMenuRemoveItemsByTitle toolsMenu "V-Ray Light Lister") do (updateEnable = true)
			if (vrayMenuRemoveItemsByTitle toolsMenu "V-Ray vrmat converter") do (updateEnable = true)
		)
	)

	local createMenuItem
	for i=1 to menuName_Create.count do (if (createMenuItem==undefined) do (createMenuItem=findMenuItem mainMenuBar menuName_Create[i]))
	if(createMenuItem!=undefined) do (
		local createMenu=createMenuItem.getSubMenu()
		local helpersMenuItem
		for i=1 to menuName_Create_Helpers.count do (if (helpersMenuItem==undefined) do (helpersMenuItem=findMenuItem createMenu menuName_Create_Helpers[i]))
		if(helpersMenuItem!=undefined) do (
			local helpersMenu=helpersMenuItem.getSubMenu()
			if (vrayMenuRemoveItemsByTitle helpersMenu "V-Ray") do (updateEnable=true)
		)

		local systemsMenuItem
		for i=1 to menuName_Create_Systems.count do (if (systemsMenuItem==undefined) do (systemsMenuItem=findMenuItem createMenu menuName_Create_Systems[i]))
		if(systemsMenuItem!=undefined) do (
			local systemsMenu=systemsMenuItem.getSubMenu()
			if (vrayMenuRemoveItemsByTitle systemsMenu "V-Ray") do (updateEnable=true)
		)
	)

	local modMenuItem
	for i=1 to menuName_Modifiers.count do (if (modMenuItem==undefined) do (modMenuItem=findMenuItem mainMenuBar menuName_Modifiers[i]))
	if (modMenuItem != undefined) do (
		local modMenu = modMenuItem.getSubMenu()
		if (vrayMenuRemoveItemsByTitle modMenu "V-Ray") do (updateEnable = true)
	)

	local renderingMenuItem = undefined
	for i=1 to mainMenuBar_Rendering.count do (if (renderingMenuItem==undefined) do (renderingMenuItem=findMenuItem mainMenuBar mainMenuBar_Rendering[i]))
	if (renderingMenuItem != undefined) do (
		local renderingMenu = renderingMenuItem.getSubMenu()
		if (vrayMenuRemoveItemsByTitle renderingMenu "Make V-Ray GPU Animation Preview") do (updateEnable = true)
		if (vrayMenuRemoveItemsByTitle renderingMenu "Make V-Ray RT Animation Preview") do (updateEnable = true)
	)

	-- unregister all V-Ray submenus
	local vrayMenu
	vrayMenu = menuMan.findMenu "Create - V-Ray Primitives"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Create - Lights.V-Ray Lights"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Create - Cameras.V-Ray Cameras"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Create - Helpers.V-Ray Helpers"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Create - Systems.V-Ray Systems"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu = menuMan.findMenu "Modifiers - V-Ray"
	if (vrayMenu != undefined) do (
		menuMan.unregisterMenu vrayMenu
		updateEnable = true
	)

	vrayMenu=getShadingViewportNitrousLabelMenu()
	if (vrayMenu!=undefined) then (
		local item=findMenuItem vrayMenu "V-Ray IPR"
		if (item!=undefined) do (
			vrayMenu.removeItem item
			updateEnable = true
		)
		item=findMenuItem vrayMenu "V-Ray GPU IPR"
		if (item!=undefined) do (
			vrayMenu.removeItem item
			updateEnable = true
		)

		-- Unregister debug shading menus
		if ((unregisterDbgShading false)) do (
			updateEnable=true
		)
	)

	-- Update menu to show all newly added items
	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

fn unregisterVRayQuadMenuItems = (
	local quadMenu=menuMan.getViewportRightClickMenu #nonePressed
	if (quadMenu!=undefined) do (
		local menu=quadMenu.getMenu 1
		if (menu!=undefined) do (
			-- remove VRayProperties
			local menuIndex=findMenuItemIdx menu "V-Ray properties"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove VRayShowVFB
			menuIndex=findMenuItemIdx menu "V-Ray VFB"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove VRaySceneConverter
			menuIndex=findMenuItemIdx menu "V-Ray scene converter"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove Bitmap2VRayHDRI texture converter
			menuIndex=findMenuItemIdx menu "V-Ray Bitmap to VRayHDRI converter"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove VRayMeshExport
			menuIndex=findMenuItemIdx menu "V-Ray mesh export"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
			-- remove vrsceneExport
			menuIndex=findMenuItemIdx menu ".vrscene exporter"
			if (menuIndex!=undefined) do (
				menu.removeItemByPosition menuIndex
			)
		)
	)
)

fn unregisterVRayEnhancedMenuBarItems = (
	local updateEnable = false
	local subMenu -- temporary submenu object
	local menuIndex -- temporary index used for item positioning calculation
	local subMenuItem
	local separatorIndex

	local mainMenuBar = menuMan.getMainMenuBar()

	--Get Language index in order to get corresponding to language, menu/item name
	local listLangInfo = sysinfo.GetMaxLanguage() -- #(9, 1, "ENU", "English (United States)", "en-US") - it is used the third element of language info list
	local languageIndex = findItem listLanguageIdxs listLangInfo[3]

	-- Remove Light Lister item
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_LightingCameras[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_LightingCameras_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMenu = subMenuItem.getSubMenu()
		if (vrayMenuRemoveItemsByTitle renderingMenu "V-Ray Light Lister") do (updateEnable = true)
	)

	-- Remove Make V-Ray GPU Animation Preview item
	subMenuItem = findMenuItem mainMenuBar mainMenuBar_Rendering[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_Rendering_Preview[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local renderingPreviewMenu = subMenuItem.getSubMenu()
		if (vrayMenuRemoveItemsByTitle renderingPreviewMenu "Make V-Ray GPU Animation Preview") do (updateEnable = true)
		if (vrayMenuRemoveItemsByTitle renderingPreviewMenu "Make V-Ray RT Animation Preview") do (updateEnable = true)
	)

	-- Remove geometry objects vray menu
	local subMenuItemObjects = findMenuItem mainMenuBar menuName_Enhanced_Objects[languageIndex]
	if (subMenuItemObjects != undefined) do (
		local mainObjMenu = subMenuItemObjects.getSubMenu()
		subMenuItem = findMenuItem mainObjMenu "V-Ray"
		if (subMenuItem != undefined) do (
			subMenu = subMenuItem.getSubMenu()

			menuIndex = findMenuItemIdx subMenu "V-Ray Proxy"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)

			menuIndex = findMenuItemIdx subMenu "V-Ray Fur"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)

			menuIndex = findMenuItemIdx subMenu "V-Ray Plane"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)

			menuIndex = findMenuItemIdx subMenu "V-Ray Sphere"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)

			menuMan.unregisterMenu subMenu
			updateEnable = true
		)
	)

	-- Remove light objects from  (Objects - Enhanced and Lighting/Cameras - Enhanced)Lights menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Lights[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local lightsMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Plane Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			if ((lightsMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				lightsMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Dome Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Sphere Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Mesh Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray IES Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Ambient Light"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx lightsMenu "V-Ray Sun"
		if (menuIndex != undefined) do (
			lightsMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
	)

	-- Remove camera objects from (Objects - Enhanced and Lighting/Cameras - Enhanced)/Cameras menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Cameras[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local camerasMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx camerasMenu "V-Ray Dome Camera"
		if (menuIndex != undefined) do (
			camerasMenu.removeItemByPosition(menuIndex)
			if ((camerasMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				camerasMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx camerasMenu "V-Ray Physical Camera"
		if (menuIndex != undefined) do (
			camerasMenu.removeItemByPosition(menuIndex)
			updateEnable = true
		)
	)


	-- Remove helper objects vray menu
	subMenuItem = undefined
	if (subMenuItemObjects != undefined) do (
		subMenu = subMenuItemObjects.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Create_Helpers[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local helpersMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx helpersMenu "V-Ray Light Meter"
		if (menuIndex != undefined) do (
			helpersMenu.removeItemByPosition(menuIndex)
			if ((helpersMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				helpersMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx helpersMenu "V-Ray BPTracer"
		if (menuIndex != undefined) do (
			helpersMenu.removeItemByPosition(menuIndex)
			if ((helpersMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				helpersMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
		menuIndex = findMenuItemIdx helpersMenu "V-Ray Stereoscopic"
		if (menuIndex != undefined) do (
			helpersMenu.removeItemByPosition(menuIndex)
			if ((helpersMenu.getItem (menuIndex-1)).getIsSeparator()) do (
				helpersMenu.removeItemByPosition(menuIndex-1)
			)
			updateEnable = true
		)
	)

	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	-- Remove modifiers vray menu
	subMenuItem = findMenuItem mainMenuBar menuName_Modifiers[languageIndex]
	if (subMenuItem != undefined) do (
		local modMenu = subMenuItem.getSubMenu()
		local subMenuItemIdx = findMenuItemIdx modMenu "V-Ray"

		if ((subMenuItemIdx != undefined) and (modMenu.getItem (subMenuItemIdx-1)).getIsSeparator()) do (
			modMenu.removeItemByPosition(subMenuItemIdx-1)
		)

		subMenuItem = findMenuItem modMenu "V-Ray"
		if (subMenuItem != undefined) do (
			subMenu = subMenuItem.getSubMenu()

			menuIndex = findMenuItemIdx subMenu "V-Ray Displacement Mod"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)
			menuIndex = findMenuItemIdx subMenu "V-Ray Ornatrix Mod"
			if (menuIndex != undefined) do (
				subMenu.removeItemByPosition(menuIndex)
			)

			menuMan.unregisterMenu subMenu
			updateEnable = true
		)
	)

	--@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	-- Remove vray vismat convertor
	subMenuItem = findMenuItem mainMenuBar menuName_Enhanced_Materials[languageIndex]
	if (subMenuItem != undefined) do (
		subMenu = subMenuItem.getSubMenu()
		subMenuItem =  findMenuItem subMenu menuName_Enhanced_Materials_Tools[languageIndex]
	)
	if (subMenuItem != undefined) do (
		local toolsMaterialMenu = subMenuItem.getSubMenu()
		menuIndex = findMenuItemIdx toolsMaterialMenu "V-Ray vrmat converter"
		if (menuIndex != undefined) do (
			toolsMaterialMenu.removeItemByPosition (menuIndex)
			updateEnable = true
		)
	)

	if (updateEnable == true) do (
		menuMan.updateMenuBar()
	)
)

fn registerVRayMenus = (
	local currentWorkspace = WorkspaceManager.GetCurrentWorkspaceIdentifier()
	if (currentWorkspace == "Workspace1") do (
		registerVRayMenuBarItems()
	)
	if (currentWorkspace == "Workspace1Enhanced") do (
		registerVRayEnhancedMenuBarItems()
	)
	registerVRayQuadMenuItems()

	menuMan.saveMenuFile (menuMan.getMenuFile())
)

fn unregisterVRayMenus = (
	local currentWorkspace = WorkspaceManager.GetCurrentWorkspaceIdentifier()
	if (currentWorkspace == "Workspace1") do (
		unregisterVRayMenuBarItems()
	)
	if (currentWorkspace == "Workspace1Enhanced") do (
		unregisterVRayEnhancedMenuBarItems()
	)
	unregisterVRayQuadMenuItems()
)

fn updateVRayMenus = (
	unregisterVRayMenus()
	registerVRayMenus()
)

fn updateVRaySceneConverterScripts = (

	if (maxver >= 19000) do (
		local string_folder = sceneconverter.ScriptFolder
		local string_originalfolder = GetDir #maxroot + "SceneConverter"

		if  false == doesfileExist string_folder then
		(
			--This folder doesn't exist yet, create it
			HiddenDOSCommand ("mkdir \""+string_folder+"\\ConversionScripts\"")
			HiddenDOSCommand ("mkdir \""+string_folder+"\\ConversionPresets\"")
		)
		--Copy the Scene converter files in UserProfileFolder ConversionScripts
		local cs_sourcepath = string_originalfolder + "\\ConversionScripts\\ChaosGroup_To_*.ms"
		local cs_destpath = string_folder + "\\ConversionScripts"
		local cmd = "copy /Y \""+cs_sourcepath+"\" \""+cs_destpath+"\""
		HiddenDOSCommand (cmd)
	)
)
